;+
;
;                      Tropospheric Emission
;                          Spectrometer
;                             (TES)
;
;                         Copyright 2008,2009,2010,2011
;              California Institute of Technology.
;                       ALL RIGHTS RESERVED.
;           U.S. Government Sponsorship acknowledged.
;
;           Any commercial use must be negotiated with the
;           Office of Technology Transfer at the California
;           Institute of Technology.
; 
;  This software may be subject to U.S. export control laws. 
;  By accepting this software, the user agrees to comply with all 
;  applicable U.S. export laws and regulations. User has the 
;  responsibility to obtain export licenses, or other export 
;  authority as may be required before exporting such information 
;  to foreign countries or providing access to foreign persons.
;
;
; read_l2_hdf
;
; PURPOSE:
;	Read a TES L2 HDF species data product file and
;	returns the data fields and geolocation for data
;	versions F01, F02, F03, F04, F05.  Data are returned in simple
;	structures of simple arrays with no processing, filtering,
;	etc.
;	Does NOT read ANCILLARY or SUMMARY files
;	IDL 6.3+ REQUIRED to read version F04+ files.
;	IDL 6.1+ REQUIRED to read version F01-F03 files
;
; CALL: 
; 	read_l2_hdf,file,species_data,geo_data $
;	  species_attributes = species_attributes, $
;	  geo_attributes = geo_attributes, $
;	  file_attributes = file_attributes,
;	  hdfeos_data = hdfeos_data, $
;	  read_ph = read_ph,status = status
;
; INPUTS:
;	file = name of file to be read (string)
;
; KEYWORD INPUT:
;	read_ph = if set, read production history.  If not set, the
;		production history will be a null string
;
;
; OUTPUTS:
;	species_data = a structure containing all of the data fields
;		       found in the species swath group.  These
;		       fields are (alphabetically except the first three):
;                species  (i.e. O3 [for ozone] in file)
;                Precision (in file this is 'species'Precision .i.e O3Precision)
;                VerticalResolution (in file this is
;                	'species'VerticalResolution,
;                	i.e O3VerticalResolution) (F01,F02)
;                	F03+-this is just 'VerticalResolution' in the
;                	file
;		 AirDensity (F03+)
;		 Altitude
;		 AverageCloudEffOpticalDepth (F03+)
;                AveragingKernel
;                AveragingKernelDiagonal (F03+)
;		 Calibration_QA (F03+)
;		 CloudEffectiveOpticalDepth (F02+)
;		 CloudEffectiveOpticalDepthError (F02+)
;                CloudTopHeight (F01)
;                CloudTopPressure
;                CloudTopPressureError (F02+)
;		 CloudVariability_QA (F03+)
;                ConstraintVector
;                DegreesOfFreedomForSignal
;                DeviationVsRetrievalCovariance
;		 H2O_HDO_Corr_QA (F04+) (Nadir Only)
;                InformationContent
;		 Initial (F03+)
;		 KDotDL_QA (F03+)
;		 LDotDL_QA (F03+)
;                MaxNumIterations
;                MeasurementErrorCovariance
;                NumberIterPerformed (in versions F01-F02 in the file this 
;                		      NumbIterPerformed, but is still
;                		      returned as NumberIterPerformed)
;		 O3_Ccurve_QA (F05+) (Nadir Only)
;		 ObservationErrorCovariance (F03+)
; FIX
;		 OzoneIRFK (F06+) (Nadir Only)
;		 OzoneTroposphericColumn (F06+) (Nadir Only)
;		 OzoneTroposphericColumnError (F06+) (Nadir Only)
;		 OzoneTroposphericColumnInitial	(F06+) (Nadir Only)
;                Pressure
;                RadianceResidualMax
;                RadianceResidualMean
;                RadianceResidualRMS
;                Scan_Averaged_Count (Nadir Only)
;                SpeciesRetrievalConverged
;                SpeciesRetrievalQuality (F02+)
;		 SurfaceEmissMean_QA (F03+)  (Nadir Only)
;		 SurfaceEmissionLayer_QA (F04)  (Nadir Only)
;		 SurfaceTempConstraint (F03+) (Nadir Only)
;		 SurfaceTempDegreesOfFreedom (F03+) (Nadir Only)
;                SurfaceTempError (Nadir Only) 
;		 SurfaceTempInitial (F03+) (Nadir Only)
;		 SurfaceTempObservationError (F03+) (Nadir Only)
;		 SurfaceTempPrecision (F03+) (Nadir Only)
;		 SurfaceTempVsApriori_QA (F03+) (Nadir Only)
;		 SurfaceTempVsAtmTemp_QA (F03+) (Nadir Only)
;                SurfaceTemperature (Nadir Only)
;                SystematicErrorCovariance (F01-F02)
;                TotalColumnDensity
;                TotalColumnDensityError
;                TotalColumnDensityInitial (F03+)
;                TotalError
;                TotalErrorCovariance
;		 TropopausePressure (F06+)
;	    	 UTCTime (F04+) 	 
;	
;		where (species) is the retrieved species or
;		temperature (O3, CH4, CO, H2O, ATM, ...)
;
;	also included in this structure are these swath level attribute
;	data fields:
;               PressureGrid (Pressure in the file)
;	  	CloudFrequency  (F02+)
;		EmissivityWavenumber (F02+)
;               VerticalCoordinate 
;               Scan_Resolution
;
;
;	geo_data = a structure containing all of the geolocation
;	fields found in the species swath group.  These fields
;	are :
;
;           BoresightAzimuth
;	    BoresightNadirAngle 
;           BoresightNadirAngleUnc
;	    BoresightTangentHeight (F03+) (Limb Only)
;	    BoresightTangentHeightUnc (F03+) (Limb Only)
;	    DayNightFlag (F03+)
;	    DominantSurfaceType (F03+)
;           Latitude
;           Latitude_Footprint_1
;           Latitude_Footprint_2
;           Latitude_Footprint_3
;           Latitude_Footprint_4
;           LocalSolarTime
;           Longitude
;           Longitude_Footprint_1
;           Longitude_Footprint_2
;           Longitude_Footprint_3
;           Longitude_Footprint_4
;	    Scan (F02+)
;	    Sequence (F02+)
;           SolarZenithAngle
;           SurfaceElevStandardDeviation
;	    SurfaceTypeFootprint (F03+) 
;           Tgt_SpacecraftAzimuth
;           Tgt_SpacecraftZenith
;           Time
;	    UTCTime (F03) Note: in the HDF file this is stored as an
;	                 27xN array (where N is the number of observations
;	                 in this file) and 27 is the number of
;	                 characters in the string.  This program concatinates
;	                 all of the characters into a single string
;	                 and returns a string array of N length.
;			This entry has no _FillValue field in its
;			attributes
;		NOTE: This field is moved out of geolocation to the
;		data fields area for version F04 

;
; KEYWORD OUTPUT:
;	species_attributes = a structure of attributes structures, with
;		one entry for each of the species data fields.  An attributes
;		structure containts the following fields:
;
;		_FillValue
;		MissingValue
;		Title
;		Units
;	 	UniqueFieldDefinition
; 
;
;	geo_attributes = structure of attributes structues, with
;		one entry for each of the geolocation data fields.  
;		An attributes structure containts the following fields:
;
;		_FillValue
;		MissingValue
;		Title
;		Units
;	 	UniqueFieldDefinition
; 
;	file_attributes = a structure containing the information
;		contained in the file attributes group in the
;		additional group.  The fields in this structure are:
;                  ProductionHistory  (not read by default--see input
;                      keywords--this is added by the TES SIPS and is thus 
;                      not present in files produced by the TES SCF)
;                  InstrumentName 
;                  ProcessLevel 
;                  TAI93AtOzOfGranule (F01- TAI93 At "capital o" z Of
;                                          Granule) (a typo)
;			(obsolete--all returned data will be
;			labeled TAI93At0zOfGranule)
;                  TAI93At0zOfGranule (F02+ TAI93 At "zero" z Of Granule)
;                  GlobalSurveyNumber
;                  GranuleMonth
;                  GranuleDay
;                  GranuleYear
;                  SurveyMode
;                  PGEVersion
;		   Command_Seq_ID (F02+)
;
;	hdfeos_data = a structure that contains the information found 
;		in the HDFEOS Information group.  The fields are:
;              		StructMetadata 
;			coremetadata
;			HDFEOSVersion
;
;	status = Contains success or failure status of the read
;		 0 for success
;		 1 if file does not exist
;		 2 the file name does not have  FXX (XX version number) in name
;		 3 the file is not HDF 5
;		 4 'Naidr' or 'Limb' not found in the file name as expected
;		 5 the file name does not have the expected .he5
;		 extension
;		 6 File is ANCILLARY or SUMMARY type file
;		 7 IDL version incompatability
;
; NOTE:  On any error except those listed, the program will stop in
; the standard IDL fashion at the error location.  No other error
; handling is built into this procedure other than those shown in the 
; status list.
;		 
; ORIGINAL AUTHOR:
;	Brendan Fisher, JPL, 08/2005
;
; UPDATE HISTORY:
;	11/2005, Brendan Fisher, JPL
;	-Changed to default to not reading production history
;	-updated to read F02 files
;	01/2006, Brendan Fisher, JPL.
;	-updated for F03 files
;	-Added Limb functionality
;	-Changed version checking to check version part of filename
;	only (ignor science content version number)
;	-changed nadir_pressure (swath level attribute pressure) to
;	pressuregrid
;	2006/04/03, Brendan Fisher, JPL, fixed bug in UTCTime
;	conversion
;	2006/08/22, Brendan Fisher, JPL, started update to F04
;	-added check to reject SUMMARY and ANCILLARY type files
;	2006/09/18, Brendan Fisher, JPL, added IDL version checking.
;	2007/12, Brendan Fisher, JPL, added F05 capability
;	2010/04, Brendan Fisher, JPL, added F06 capability
;-



pro read_l2_hdf,file,species_data,geo_data,status=status, $
  species_attributes = species_attributes,geo_attributes = geo_attributes, $
  additional_data = additional_data, hdfeos_data = hdfeos_data,read_ph=read_ph


status=0
if n_elements(read_ph) eq 0 then read_ph = 0

; check for file existence
ff = file_search(file,count=count)
if count eq 0 then begin
    print,'Cannot find file:'
    print,file
    status = 1
    return
endif
; check type
apos = strpos(file,'ANCILLARY')
spos = strpos(file,'SUMMARY')
supos = strpos(file,'SUPPLEMENTAL')
if apos ne -1 or spos ne -1 or supos ne -1 then begin
    print,'This reader (read_l2_hdf) does not work for ANCILLARY, SUMMARY, or SUPPLEMENTAL product files.'
    print,'File name: ',file
    status=6
    return
end
; check version
pos1 = strpos(file,'_F01_')
pos2 = strpos(file,'_F02_')
pos3 = strpos(file,'_F03_')
pos4 = strpos(file,'_F04_')
pos5 = strpos(file,'_F05_')
pos6 = strpos(file,'_F06_')
if pos1 eq -1 and pos2 eq -1 and pos3 eq -1 and $
  pos4 eq -1 and pos5 eq -1 and pos6 eq -1 then begin
    print,'Expected version _F01_, _F02_, _F03_, _F04_, _F05 or _F06_ in file name not found.'
    print,file
    print,'This file format version not supported.'
    status = 2
    return
endif
if pos1 ne -1 then version = 1
if pos2 ne -1 then version = 2
if pos3 ne -1 then version = 3
if pos4 ne -1 then version = 4
if pos5 ne -1 then version = 5
if pos6 ne -1 then version = 6
; check to see if file is an HDF5 
if not h5f_is_hdf5(file) then begin
    print,'File is not HDF 5:'
    print,file
    status=3
    return
endif
; check for Nadir or Limb in name
posn = strpos(file,'Nadir')
posl = strpos(file,'Limb')
if posn eq -1 and posl eq -1 then begin
    print,'Expected Nadir or Limb in file name not found.'
    print,file
    status = 4
    return
endif
if posn ne -1 then type = 'Nadir'
if posl ne -1 then type = 'Limb'
;if type eq 'Limb' then version = 3
;check for .he5 extension
;
len = strlen(file)
exten = strmid(file,len-3,3)
if exten ne 'he5' then begin
    print,'Expected .he5 file name extension not found.'
    print,file
    status=5
    return
endif
; IDL version checking
iversion = !version
idl_version = float(iversion.release)
if version ge 4 and idl_version lt 6.3 then begin
    print,'IDL v6.3+ required for TES format version F04+ files.'
    status = 7
    return
endif
if idl_version lt 6.1 and version le 3 then begin
    print,'IDL v6.1+ required for TES format version F01, F02, and F03 files.'
    status = 7
    return
endif
;
; get species from file name
;
; file name always has '_L2-species-'
;
; The new ASDC stuff is breaking our file naming, so a new way of getting
; species name is needed.
;
; All files manes have the this format:
;
; TES-Aura_L2-species-  here 
; TES_Aura_L2_species_  at ASDC
;
pos = strpos(file,'TES-Aura_L2-')
if pos eq -1 then pos = strpos(file,'TES_Aura_L2_')
pos2 = strpos(file,'-',pos+12)
if pos2 eq -1 then pos2 =  strpos(file,'_',pos+12)
gas=strmid(file,pos+12,pos2-(pos+12))
if gas eq 'ATM' then gas = 'TATM'
;print,gas


;
; open file to data fields
;
fid = h5f_open(file)
hdfid = h5g_open(fid,'HDFEOS')
swid = h5g_open(hdfid,'SWATHS')
specid = h5g_open(swid,gas+type+'Swath')
; data fields
fieldid = h5g_open(specid,'Data Fields')
;
; actual data, alphabetically except for the gas, gasPrecision, and
; gasVerticalResolution
;
dataid = h5d_open(fieldid,gas)
species = h5d_read(dataid)
get_data_attributes,dataid,size(species,/type),attrib0
h5d_close,dataid

dataid = h5d_open(fieldid,gas+'Precision')
Precision = h5d_read(dataid)
get_data_attributes,dataid,size(Precision,/type),attrib1
h5d_close,dataid

if version le 2 then begin
    dataid = h5d_open(fieldid,gas+'VerticalResolution')
    VerticalResolution = h5d_read(dataid)
    get_data_attributes,dataid,size(VerticalResolution,/type),attrib2
    h5d_close,dataid
endif 

if version ge 3 then begin
    dataid = h5d_open(fieldid,'VerticalResolution')
    VerticalResolution = h5d_read(dataid)
    get_data_attributes,dataid,size(VerticalResolution,/type),attrib2
    h5d_close,dataid
endif

; the rest, alphabetically

if version ge 3 then begin
    dataid = h5d_open(fieldid,'AirDensity')
    AirDensity = h5d_read(dataid)
    get_data_attributes,dataid,size(AirDensity,/type),attrib3
    h5d_close,dataid
endif

dataid = h5d_open(fieldid,'Altitude')
altitude = h5d_read(dataid)
get_data_attributes,dataid,size(altitude,/type),attrib4
h5d_close,dataid

if version ge 3 then begin
    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepth')
    AverageCloudEffOpticalDepth = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepth,/type),attrib5
    h5d_close,dataid
endif


if version ge 5 then begin
    dataid = h5d_open(fieldid,'AverageCloudEffOpticalDepthError')
    AverageCloudEffOpticalDepthError = h5d_read(dataid)
    get_data_attributes,dataid,size(AverageCloudEffOpticalDepth,/type),attrib5a
    h5d_close,dataid
endif

dataid = h5d_open(fieldid,'AveragingKernel')
Averagingkernel = h5d_read(dataid)
get_data_attributes,dataid,size(AveragingKernel,/type),attrib6
h5d_close,dataid

if version ge 3 then begin
    dataid = h5d_open(fieldid,'AveragingKernelDiagonal')
    AveragingKernelDiagonal = h5d_read(dataid)
    get_data_attributes,dataid,size(AveragingKernelDiagonal,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(fieldid,'Calibration_QA')
    Calibration_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(Calibration_QA,/type),attrib8
    h5d_close,dataid
endif

if version ge 2 then begin
    dataid = h5d_open(fieldid,'CloudEffectiveOpticalDepth')
    CloudEffectiveOpticalDepth = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudEffectiveOpticalDepth,/type),attrib9
    h5d_close,dataid

    dataid = h5d_open(fieldid,'CloudEffectiveOpticalDepthError')
    CloudEffectiveOpticalDepthError = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudEffectiveOpticalDepthError,/type), $
                        attrib10
    h5d_close,dataid
endif

if version eq 1 then begin
    dataid = h5d_open(fieldid,'CloudTopHeight')
    CloudTopHeight = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopHeight,/type),attrib11
    h5d_close,dataid
endif

dataid = h5d_open(fieldid,'CloudTopPressure')
CloudTopPressure = h5d_read(dataid)
get_data_attributes,dataid,size(CloudTopPressure,/type),attrib12
h5d_close,dataid

if version ge 2 then begin
    dataid = h5d_open(fieldid,'CloudTopPressureError')
    CloudTopPressureError = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudTopPressureError,/type),attrib13
    h5d_close,dataid
endif

if version ge 3 then begin
    dataid = h5d_open(fieldid,'CloudVariability_QA')
    CloudVariability_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(CloudVariability_QA,/type),attrib14
    h5d_close,dataid
endif

dataid = h5d_open(fieldid,'ConstraintVector')
ConstraintVector = h5d_read(dataid)
get_data_attributes,dataid,size(ConstraintVector,/type),attrib15
h5d_close,dataid

dataid = h5d_open(fieldid,'DegreesOfFreedomForSignal')
DegreesOfFreedomForSignal = h5d_read(dataid)
get_data_attributes,dataid,size(DegreesOfFreedomForSignal,/type),attrib16
h5d_close,dataid

dataid = h5d_open(fieldid,'DeviationVsRetrievalCovariance')
DeviationVsRetrievalCovariance = h5d_read(dataid)
get_data_attributes,dataid,size(DeviationVsRetrievalCovariance,/type),attrib17
h5d_close,dataid

if version ge 4 then begin
    dataid = h5d_open(fieldid,'H2O_HDO_Corr_QA')
    H2O_HDO_Corr_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(H2O_HDO_Corr_QA,/type),attrib51
    h5d_close,dataid
endif

dataid = h5d_open(fieldid,'InformationContent')
InformationContent = h5d_read(dataid)
get_data_attributes,dataid,size(InformationContent,/type),attrib18
h5d_close,dataid

if version ge 3 then begin
    dataid = h5d_open(fieldid,'Initial')
    Initial = h5d_read(dataid)
    get_data_attributes,dataid,size(Initial,/type),attrib19
    h5d_close,dataid

    dataid = h5d_open(fieldid,'KDotDL_QA')
    KDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(KDotDL_QA,/type),attrib20
    h5d_close,dataid

    dataid = h5d_open(fieldid,'LDotDL_QA')
    LDotDL_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(LDotDL_QA,/type),attrib21
    h5d_close,dataid
endif

dataid = h5d_open(fieldid,'MaxNumIterations')
MaxNumIterations = h5d_read(dataid)
get_data_attributes,dataid,size(MaxNumIterations,/type),attrib22
h5d_close,dataid

dataid = h5d_open(fieldid,'MeasurementErrorCovariance')
MeasurementErrorCovariance = h5d_read(dataid)
get_data_attributes,dataid,size(MeasurementErrorCovariance,/type),attrib23
h5d_close,dataid

if version le 2 then begin
    dataid = h5d_open(fieldid,'NumbIterPerformed')
    NumberIterPerformed = h5d_read(dataid)
    get_data_attributes,dataid,size(NumberIterPerformed,/type),attrib24
    h5d_close,dataid
endif

if version ge 3 then begin
    dataid = h5d_open(fieldid,'NumberIterPerformed')
    NumberIterPerformed = h5d_read(dataid)
    get_data_attributes,dataid,size(NumberIterPerformed,/type),attrib24
    h5d_close,dataid
endif

if version ge 3 then begin
    dataid = h5d_open(fieldid,'ObservationErrorCovariance')
    ObservationErrorCovariance = h5d_read(dataid)
    get_data_attributes,dataid,size(ObservationErrorCovariance,/type),attrib25
    h5d_close,dataid

endif

if version ge 5 and type eq 'Nadir' then begin
    dataid = h5d_open(fieldid,'O3_Ccurve_QA')
    O3_Ccurve_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(O3_Ccurve_QA,/type),attrib26a
    h5d_close,dataid
endif


if version ge 6 and type eq 'Nadir' then begin 

    dataid = h5d_open(fieldid,'OzoneIRFK')
    OzoneIRFK = h5d_read(dataid)
    get_data_attributes,dataid,size(OzoneIRFK,/type),attrib26e
    h5d_close,dataid

    dataid = h5d_open(fieldid,'OzoneTroposphericColumn')
    OzoneTroposphericColumn = h5d_read(dataid)
    get_data_attributes,dataid,size(OzoneTroposphericColumn,/type),attrib26b
    h5d_close,dataid

    dataid = h5d_open(fieldid,'OzoneTroposphericColumnError')
    OzoneTroposphericColumnError = h5d_read(dataid)
    get_data_attributes,dataid,size(OzoneTroposphericColumnError,/type),attrib26c
    h5d_close,dataid

    dataid = h5d_open(fieldid,'OzoneTroposphericColumnInitial')
    OzoneTroposphericColumnInitial = h5d_read(dataid)
    get_data_attributes,dataid,size(OzoneTroposphericColumnInitial,/type),attrib26d
    h5d_close,dataid
endif
 


dataid = h5d_open(fieldid,'Pressure')
Pressure = h5d_read(dataid)
get_data_attributes,dataid,size(Pressure,/type),attrib26
h5d_close,dataid

dataid = h5d_open(fieldid,'RadianceResidualMax')
RadianceResidualMax = h5d_read(dataid)
get_data_attributes,dataid,size(RadianceResidualMax,/type),attrib27
h5d_close,dataid

dataid = h5d_open(fieldid,'RadianceResidualMean')
RadianceResidualMean = h5d_read(dataid)
get_data_attributes,dataid,size(RadianceResidualMean,/type),attrib28
h5d_close,dataid

dataid = h5d_open(fieldid,'RadianceResidualRMS')
RadianceResidualRMS = h5d_read(dataid)
get_data_attributes,dataid,size(RadianceResidualRMS,/type),attrib29
h5d_close,dataid

if type eq 'Nadir' then begin
    dataid = h5d_open(fieldid,'Scan_Averaged_Count')
    Scan_Averaged_Count = h5d_read(dataid)
    get_data_attributes,dataid,size(Scan_Averaged_Count,/type),attrib30
    h5d_close,dataid
endif

dataid = h5d_open(fieldid,'SpeciesRetrievalConverged')
SpeciesRetrievalConverged = h5d_read(dataid)
get_data_attributes,dataid,size(SpeciesRetrievalConverged,/type),attrib31
h5d_close,dataid

if version ge 2 then begin
    dataid = h5d_open(fieldid,'SpeciesRetrievalQuality')
    SpeciesRetrievalQuality = h5d_read(dataid)
    get_data_attributes,dataid,size(SpeciesRetrievalQuality,/type),attrib32
    h5d_close,dataid
endif

if version ge 3 and type eq 'Nadir' then begin
    dataid = h5d_open(fieldid,'SurfaceEmissMean_QA')
    SurfaceEmissMean_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceEmissMean_QA,/type),attrib33
    h5d_close,dataid
endif

if version ge 4 and type eq 'Nadir' then begin
    dataid = h5d_open(fieldid,'SurfaceEmissionLayer_QA')
    SurfaceEmissionLayer_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceEmissionLatyer_QA,/type),attrib50
    h5d_close,dataid
endif

if version ge 3 and type eq 'Nadir' then begin
    dataid = h5d_open(fieldid,'SurfaceTempConstraint')
    SurfaceTempConstraint = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempConstraint,/type),attrib34
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempDegreesOfFreedom')
    SurfaceTempDegreesOfFreedom = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempDegreesOfFreedom,/type),attrib35
    h5d_close,dataid
endif

if type eq 'Nadir' then begin
    dataid = h5d_open(fieldid,'SurfaceTempError')
    SurfaceTempError = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempError,/type),attrib36
    h5d_close,dataid
endif

if version ge 3 and type eq 'Nadir' then begin
    dataid = h5d_open(fieldid,'SurfaceTempInitial')
    SurfaceTempInitial = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempInitial,/type),attrib37
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempObservationError')
    SurfaceTempObservationError = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempObservationError,/type),attrib38
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempPrecision')
    SurfaceTempPrecision = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempPrecision,/type),attrib39
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempVsApriori_QA')
    SurfaceTempVsApriori_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempVsApriori_QA,/type),attrib40
    h5d_close,dataid

    dataid = h5d_open(fieldid,'SurfaceTempVsAtmTemp_QA')
    SurfaceTempVsAtmTemp_QA = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTempVsAtmTemp_QA,/type),attrib41
    h5d_close,dataid
endif

if type eq 'Nadir' then begin
    dataid = h5d_open(fieldid,'SurfaceTemperature')
    SurfaceTemperature = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTemperature,/type),attrib42
    h5d_close,dataid
endif

if version le 2 then begin
    dataid = h5d_open(fieldid,'SystematicErrorCovariance')
    SystematicErrorCovariance = h5d_read(dataid)
    get_data_attributes,dataid,size(SystematicErrorCovariance,/type),attrib43
    h5d_close,dataid
endif

dataid = h5d_open(fieldid,'TotalColumnDensity')
TotalColumnDensity = h5d_read(dataid)
get_data_attributes,dataid,size(TotalColumnDensity,/type),attrib44
h5d_close,dataid

dataid = h5d_open(fieldid,'TotalColumnDensityError')
TotalColumnDensityError = h5d_read(dataid)
get_data_attributes,dataid,size(TotalColumnDensityError,/type),attrib45
h5d_close,dataid

if version ge 3 then begin
    dataid = h5d_open(fieldid,'TotalColumnDensityInitial')
    TotalColumnDensityInitial = h5d_read(dataid)
    get_data_attributes,dataid,size(TotalColumnDensityInitial,/type),attrib46
    h5d_close,dataid
endif

dataid = h5d_open(fieldid,'TotalError')
TotalError = h5d_read(dataid)
get_data_attributes,dataid,size(TotalError,/type),attrib47
h5d_close,dataid

dataid = h5d_open(fieldid,'TotalErrorCovariance')
TotalErrorCovariance = h5d_read(dataid)
get_data_attributes,dataid,size(TotalErrorCovariance,/type),attrib48
h5d_close,dataid

if version ge 6 then begin
    dataid = h5d_open(fieldid,'TropopausePressure')
    TropopausePressure = h5d_read(dataid)
    get_data_attributes,dataid,size(TropopausePressure,/type),attrib48a
    h5d_close,dataid
endif


if version ge 4 then begin
    dataid = h5d_open(fieldid,'UTCTime')
    UTCTime = h5d_read(dataid)
    get_data_attributes,dataid,size(utctime,/type),attrib49,/utctime
    h5d_close,dataid
endif

h5g_close,fieldid




;
; get swath level attributes
;
id = h5a_open_name(specid,'Pressure')
pressuregrid = h5a_read(id)
h5a_close,id

id = h5a_open_name(specid,'VerticalCoordinate')
VerticalCoordinate = h5a_read(id)
h5a_close,id

if version ge 2 then begin
    id = h5a_open_name(specid,'CloudFrequency')
    CloudFrequency = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'EmissivityWavenumber')
    EmissivityWavenumber = h5a_read(id)
    h5a_close,id
endif

id = h5a_open_name(specid,'Scan_Resolution')
Scan_Resolution = h5a_read(id)
h5a_close,id



;
; put all species data into return structure
;
if version eq 1 then begin
    species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               VerticalResolution : temporary(VerticalResolution), $
               Altitude : temporary(Altitude), $
               AveragingKernel : temporary(Averagingkernel), $
               CloudTopHeight : temporary(CloudTopHeight), $
               CloudTopPressure : temporary(CloudTopPressure), $
               ConstraintVector : temporary(ConstraintVector), $
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               InformationContent : temporary(InformationContent), $
               MaxNumIterations : temporary(MaxNumIterations), $
               MeasurementErrorCovariance : temporary(MeasurementErrorCovariance), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               Scan_Averaged_Count : temporary(Scan_Averaged_Count), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SurfaceTempError : temporary(SurfaceTempError), $
               SurfaceTemperature : temporary(SurfaceTemperature), $
               SystematicErrorCovariance : temporary(SystematicErrorCovariance), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalError : temporary(TotalError), $
               TotalErrorCovariance : temporary(TotalErrorCovariance), $
               pressuregrid : temporary(pressuregrid), $
               VerticalCoordinate : temporary(VerticalCoordinate), $
               Scan_Resolution : temporary(Scan_Resolution) $
               }
endif 
if version eq 2 then begin
    species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               VerticalResolution : temporary(VerticalResolution), $
               Altitude : temporary(Altitude), $
               AveragingKernel : temporary(Averagingkernel), $
               CloudEffectiveOpticalDepth : temporary(CloudEffectiveOpticalDepth), $
               CloudEffectiveOpticalDepthError : temporary(CloudEffectiveOpticalDepthError), $
               CloudTopPressure : temporary(CloudTopPressure), $
               CloudTopPressureError : temporary(CloudTopPressureError), $
               ConstraintVector : temporary(ConstraintVector), $
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               InformationContent : temporary(InformationContent), $
               MaxNumIterations : temporary(MaxNumIterations), $
               MeasurementErrorCovariance : temporary(MeasurementErrorCovariance), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               Scan_Averaged_Count : temporary(Scan_Averaged_Count), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SpeciesRetrievalQuality : temporary(SpeciesRetrievalQuality), $
               SurfaceTempError : temporary(SurfaceTempError), $
               SurfaceTemperature : temporary(SurfaceTemperature), $
               SystematicErrorCovariance : temporary(SystematicErrorCovariance), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalError : temporary(TotalError), $
               TotalErrorCovariance : temporary(TotalErrorCovariance), $
               CloudFrequency : temporary(CloudFrequency),$
               EmissivityWavenumber : temporary(EmissivityWavenumber), $
               pressuregrid : temporary(pressuregrid), $
               VerticalCoordinate : temporary(VerticalCoordinate), $
               Scan_Resolution : temporary(Scan_Resolution) $
               }
endif 
;stop

if version eq 3 and type eq 'Nadir' then begin
    species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               VerticalResolution : temporary(VerticalResolution), $
               AirDensity : temporary(AirDensity), $    
               Altitude : temporary(Altitude), $
               AverageCloudEffOpticalDepth : temporary(AverageCloudEffOpticalDepth), $    
               AveragingKernel : temporary(Averagingkernel), $
               AveragingKernelDiagonal : temporary(AveragingKernelDiagonal), $    
               Calibration_QA : temporary(Calibration_QA), $
               CloudEffectiveOpticalDepth : temporary(CloudEffectiveOpticalDepth), $
               CloudEffectiveOpticalDepthError : temporary(CloudEffectiveOpticalDepthError), $
               CloudTopPressure : temporary(CloudTopPressure), $
               CloudTopPressureError : temporary(CloudTopPressureError), $
               CloudVariability_QA : temporary(CloudVariability_QA), $    
               ConstraintVector : temporary(ConstraintVector), $
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               InformationContent : temporary(InformationContent), $
               Initial : temporary(Initial), $    
               KDotDL_QA : temporary(KDotDL_QA), $    
               LDotDL_QA : temporary(LDotDL_QA), $    
               MaxNumIterations : temporary(MaxNumIterations), $
               MeasurementErrorCovariance : temporary(MeasurementErrorCovariance), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               ObservationErrorCovariance : temporary(ObservationErrorCovariance), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               Scan_Averaged_Count : temporary(Scan_Averaged_Count), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SpeciesRetrievalQuality : temporary(SpeciesRetrievalQuality), $
               SurfaceEmissMean_QA : temporary(SurfaceEmissMean_QA), $    
               SurfaceTempConstraint : temporary(SurfaceTempConstraint), $    
               SurfaceTempDegreesOfFreedom : temporary(SurfaceTempDegreesOfFreedom), $    
               SurfaceTempError : temporary(SurfaceTempError), $
               SurfaceTempInitial : temporary(SurfaceTempInitial), $    
               SurfaceTempObservationError : temporary(SurfaceTempObservationError), $    
               SurfaceTempPrecision : temporary(SurfaceTempPrecision), $    
               SurfaceTempVsApriori_QA : temporary(SurfaceTempVsApriori_QA), $    
               SurfaceTempVsAtmTemp_QA : temporary(SurfaceTempVsAtmTemp_QA), $    
               SurfaceTemperature : temporary(SurfaceTemperature), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalColumnDensityInitial : temporary(TotalColumnDensityInitial), $    
               TotalError : temporary(TotalError), $
               TotalErrorCovariance : temporary(TotalErrorCovariance), $
               CloudFrequency : temporary(CloudFrequency),$
               EmissivityWavenumber : temporary(EmissivityWavenumber), $
               pressuregrid : temporary(pressuregrid), $
               VerticalCoordinate : temporary(VerticalCoordinate), $
               Scan_Resolution : temporary(Scan_Resolution) $  
               }
endif 
if version eq 3 and type eq 'Limb' then begin
    species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               VerticalResolution : temporary(VerticalResolution), $
               AirDensity : temporary(AirDensity), $    
               Altitude : temporary(Altitude), $
               AverageCloudEffOpticalDepth : temporary(AverageCloudEffOpticalDepth), $    
               AveragingKernel : temporary(Averagingkernel), $
               AveragingKernelDiagonal : temporary(AveragingKernelDiagonal), $    
               Calibration_QA : temporary(Calibration_QA), $
               CloudEffectiveOpticalDepth : temporary(CloudEffectiveOpticalDepth), $
               CloudEffectiveOpticalDepthError : temporary(CloudEffectiveOpticalDepthError), $
               CloudTopPressure : temporary(CloudTopPressure), $
               CloudTopPressureError : temporary(CloudTopPressureError), $
               CloudVariability_QA : temporary(CloudVariability_QA), $    
               ConstraintVector : temporary(ConstraintVector), $
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               InformationContent : temporary(InformationContent), $
               Initial : temporary(Initial), $    
               KDotDL_QA : temporary(KDotDL_QA), $    
               LDotDL_QA : temporary(LDotDL_QA), $    
               MaxNumIterations : temporary(MaxNumIterations), $
               MeasurementErrorCovariance : temporary(MeasurementErrorCovariance), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               ObservationErrorCovariance : temporary(ObservationErrorCovariance), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SpeciesRetrievalQuality : temporary(SpeciesRetrievalQuality), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalColumnDensityInitial : temporary(TotalColumnDensityInitial), $    
               TotalError : temporary(TotalError), $
               TotalErrorCovariance : temporary(TotalErrorCovariance), $
               CloudFrequency : temporary(CloudFrequency),$
               EmissivityWavenumber : temporary(EmissivityWavenumber), $
               pressuregrid : temporary(pressuregrid), $
               VerticalCoordinate : temporary(VerticalCoordinate), $
               Scan_Resolution : temporary(Scan_Resolution) $  
               }
endif

if version eq 4 and type eq 'Nadir' then begin
    species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               VerticalResolution : temporary(VerticalResolution), $
               AirDensity : temporary(AirDensity), $    
               Altitude : temporary(Altitude), $
               AverageCloudEffOpticalDepth : temporary(AverageCloudEffOpticalDepth), $    
               AveragingKernel : temporary(Averagingkernel), $
               AveragingKernelDiagonal : temporary(AveragingKernelDiagonal), $    
               Calibration_QA : temporary(Calibration_QA), $
               CloudEffectiveOpticalDepth : temporary(CloudEffectiveOpticalDepth), $
               CloudEffectiveOpticalDepthError : temporary(CloudEffectiveOpticalDepthError), $
               CloudTopPressure : temporary(CloudTopPressure), $
               CloudTopPressureError : temporary(CloudTopPressureError), $
               CloudVariability_QA : temporary(CloudVariability_QA), $    
               ConstraintVector : temporary(ConstraintVector), $
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               H2O_HDO_Corr_QA : temporary(H2O_HDO_Corr_QA), $
               InformationContent : temporary(InformationContent), $
               Initial : temporary(Initial), $    
               KDotDL_QA : temporary(KDotDL_QA), $    
               LDotDL_QA : temporary(LDotDL_QA), $    
               MaxNumIterations : temporary(MaxNumIterations), $
               MeasurementErrorCovariance : temporary(MeasurementErrorCovariance), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               ObservationErrorCovariance : temporary(ObservationErrorCovariance), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               Scan_Averaged_Count : temporary(Scan_Averaged_Count), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SpeciesRetrievalQuality : temporary(SpeciesRetrievalQuality), $
               SurfaceEmissMean_QA : temporary(SurfaceEmissMean_QA), $    
               SurfaceEmissionLayer_QA : temporary(SurfaceEmissionLayer_QA), $    
               SurfaceTempConstraint : temporary(SurfaceTempConstraint), $    
               SurfaceTempDegreesOfFreedom : temporary(SurfaceTempDegreesOfFreedom), $    
               SurfaceTempError : temporary(SurfaceTempError), $
               SurfaceTempInitial : temporary(SurfaceTempInitial), $    
               SurfaceTempObservationError : temporary(SurfaceTempObservationError), $    
               SurfaceTempPrecision : temporary(SurfaceTempPrecision), $    
               SurfaceTempVsApriori_QA : temporary(SurfaceTempVsApriori_QA), $    
               SurfaceTempVsAtmTemp_QA : temporary(SurfaceTempVsAtmTemp_QA), $    
               SurfaceTemperature : temporary(SurfaceTemperature), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalColumnDensityInitial : temporary(TotalColumnDensityInitial), $    
               TotalError : temporary(TotalError), $
               TotalErrorCovariance : temporary(TotalErrorCovariance), $
               UTCTime : temporary(UTCTime), $
               CloudFrequency : temporary(CloudFrequency),$
               EmissivityWavenumber : temporary(EmissivityWavenumber), $
               pressuregrid : temporary(pressuregrid), $
               VerticalCoordinate : temporary(VerticalCoordinate), $
               Scan_Resolution : temporary(Scan_Resolution) $  
               }
endif 


if version eq 4 and type eq 'Limb' then begin
    species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               VerticalResolution : temporary(VerticalResolution), $
               AirDensity : temporary(AirDensity), $    
               Altitude : temporary(Altitude), $
               AverageCloudEffOpticalDepth : temporary(AverageCloudEffOpticalDepth), $    
               AveragingKernel : temporary(Averagingkernel), $
               AveragingKernelDiagonal : temporary(AveragingKernelDiagonal), $    
               Calibration_QA : temporary(Calibration_QA), $
               CloudEffectiveOpticalDepth : temporary(CloudEffectiveOpticalDepth), $
               CloudEffectiveOpticalDepthError : temporary(CloudEffectiveOpticalDepthError), $
               CloudTopPressure : temporary(CloudTopPressure), $
               CloudTopPressureError : temporary(CloudTopPressureError), $
               CloudVariability_QA : temporary(CloudVariability_QA), $    
               ConstraintVector : temporary(ConstraintVector), $
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               H2O_HDO_Corr_QA : temporary(H2O_HDO_Corr_QA), $
               InformationContent : temporary(InformationContent), $
               Initial : temporary(Initial), $    
               KDotDL_QA : temporary(KDotDL_QA), $    
               LDotDL_QA : temporary(LDotDL_QA), $    
               MaxNumIterations : temporary(MaxNumIterations), $
               MeasurementErrorCovariance : temporary(MeasurementErrorCovariance), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               ObservationErrorCovariance : temporary(ObservationErrorCovariance), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SpeciesRetrievalQuality : temporary(SpeciesRetrievalQuality), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalColumnDensityInitial : temporary(TotalColumnDensityInitial), $    
               TotalError : temporary(TotalError), $
               TotalErrorCovariance : temporary(TotalErrorCovariance), $
               UTCTime : temporary(UTCTime), $
               CloudFrequency : temporary(CloudFrequency),$
               EmissivityWavenumber : temporary(EmissivityWavenumber), $
               pressuregrid : temporary(pressuregrid), $
               VerticalCoordinate : temporary(VerticalCoordinate), $
               Scan_Resolution : temporary(Scan_Resolution) $  
               }
endif 

if version eq 5 and type eq 'Nadir' then begin
    species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               VerticalResolution : temporary(VerticalResolution), $
               AirDensity : temporary(AirDensity), $    
               Altitude : temporary(Altitude), $
               AverageCloudEffOpticalDepth : temporary(AverageCloudEffOpticalDepth), $    
               AverageCloudEffOpticalDepthError : temporary(AverageCloudEffOpticalDepthError), $    
               AveragingKernel : temporary(Averagingkernel), $
               AveragingKernelDiagonal : temporary(AveragingKernelDiagonal), $    
               Calibration_QA : temporary(Calibration_QA), $
               CloudEffectiveOpticalDepth : temporary(CloudEffectiveOpticalDepth), $
               CloudEffectiveOpticalDepthError : temporary(CloudEffectiveOpticalDepthError), $
               CloudTopPressure : temporary(CloudTopPressure), $
               CloudTopPressureError : temporary(CloudTopPressureError), $
               CloudVariability_QA : temporary(CloudVariability_QA), $    
               ConstraintVector : temporary(ConstraintVector), $
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               H2O_HDO_Corr_QA : temporary(H2O_HDO_Corr_QA), $
               InformationContent : temporary(InformationContent), $
               Initial : temporary(Initial), $    
               KDotDL_QA : temporary(KDotDL_QA), $    
               LDotDL_QA : temporary(LDotDL_QA), $    
               MaxNumIterations : temporary(MaxNumIterations), $
               MeasurementErrorCovariance : temporary(MeasurementErrorCovariance), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               ObservationErrorCovariance : temporary(ObservationErrorCovariance), $
               O3_Ccurve_QA : temporary(O3_Ccurve_QA), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               Scan_Averaged_Count : temporary(Scan_Averaged_Count), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SpeciesRetrievalQuality : temporary(SpeciesRetrievalQuality), $
               SurfaceEmissMean_QA : temporary(SurfaceEmissMean_QA), $    
               SurfaceEmissionLayer_QA : temporary(SurfaceEmissionLayer_QA), $    
               SurfaceTempConstraint : temporary(SurfaceTempConstraint), $    
               SurfaceTempDegreesOfFreedom : temporary(SurfaceTempDegreesOfFreedom), $    
               SurfaceTempError : temporary(SurfaceTempError), $
               SurfaceTempInitial : temporary(SurfaceTempInitial), $    
               SurfaceTempObservationError : temporary(SurfaceTempObservationError), $    
               SurfaceTempPrecision : temporary(SurfaceTempPrecision), $    
               SurfaceTempVsApriori_QA : temporary(SurfaceTempVsApriori_QA), $    
               SurfaceTempVsAtmTemp_QA : temporary(SurfaceTempVsAtmTemp_QA), $    
               SurfaceTemperature : temporary(SurfaceTemperature), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalColumnDensityInitial : temporary(TotalColumnDensityInitial), $    
               TotalError : temporary(TotalError), $
               TotalErrorCovariance : temporary(TotalErrorCovariance), $
               UTCTime : temporary(UTCTime), $
               CloudFrequency : temporary(CloudFrequency),$
               EmissivityWavenumber : temporary(EmissivityWavenumber), $
               pressuregrid : temporary(pressuregrid), $
               VerticalCoordinate : temporary(VerticalCoordinate), $
               Scan_Resolution : temporary(Scan_Resolution) $  
               }
endif 
if version eq 5 and type eq 'Limb' then begin
    species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               VerticalResolution : temporary(VerticalResolution), $
               AirDensity : temporary(AirDensity), $    
               Altitude : temporary(Altitude), $
               AverageCloudEffOpticalDepth : temporary(AverageCloudEffOpticalDepth), $    
               AverageCloudEffOpticalDepthError : temporary(AverageCloudEffOpticalDepthError), $    
               AveragingKernel : temporary(Averagingkernel), $
               AveragingKernelDiagonal : temporary(AveragingKernelDiagonal), $    
               Calibration_QA : temporary(Calibration_QA), $
               CloudEffectiveOpticalDepth : temporary(CloudEffectiveOpticalDepth), $
               CloudEffectiveOpticalDepthError : temporary(CloudEffectiveOpticalDepthError), $
               CloudTopPressure : temporary(CloudTopPressure), $
               CloudTopPressureError : temporary(CloudTopPressureError), $
               CloudVariability_QA : temporary(CloudVariability_QA), $    
               ConstraintVector : temporary(ConstraintVector), $
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               H2O_HDO_Corr_QA : temporary(H2O_HDO_Corr_QA), $
               InformationContent : temporary(InformationContent), $
               Initial : temporary(Initial), $    
               KDotDL_QA : temporary(KDotDL_QA), $    
               LDotDL_QA : temporary(LDotDL_QA), $    
               MaxNumIterations : temporary(MaxNumIterations), $
               MeasurementErrorCovariance : temporary(MeasurementErrorCovariance), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               ObservationErrorCovariance : temporary(ObservationErrorCovariance), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SpeciesRetrievalQuality : temporary(SpeciesRetrievalQuality), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalColumnDensityInitial : temporary(TotalColumnDensityInitial), $    
               TotalError : temporary(TotalError), $
               TotalErrorCovariance : temporary(TotalErrorCovariance), $
               UTCTime : temporary(UTCTime), $
               CloudFrequency : temporary(CloudFrequency),$
               EmissivityWavenumber : temporary(EmissivityWavenumber), $
               pressuregrid : temporary(pressuregrid), $
               VerticalCoordinate : temporary(VerticalCoordinate), $
               Scan_Resolution : temporary(Scan_Resolution) $  
               }
endif 
if version eq 6 and type eq 'Nadir' then begin
    species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               VerticalResolution : temporary(VerticalResolution), $
               AirDensity : temporary(AirDensity), $    
               Altitude : temporary(Altitude), $
               AverageCloudEffOpticalDepth : temporary(AverageCloudEffOpticalDepth), $    
               AverageCloudEffOpticalDepthError : temporary(AverageCloudEffOpticalDepthError), $    
               AveragingKernel : temporary(Averagingkernel), $
               AveragingKernelDiagonal : temporary(AveragingKernelDiagonal), $    
               Calibration_QA : temporary(Calibration_QA), $
               CloudEffectiveOpticalDepth : temporary(CloudEffectiveOpticalDepth), $
               CloudEffectiveOpticalDepthError : temporary(CloudEffectiveOpticalDepthError), $
               CloudTopPressure : temporary(CloudTopPressure), $
               CloudTopPressureError : temporary(CloudTopPressureError), $
               CloudVariability_QA : temporary(CloudVariability_QA), $    
               ConstraintVector : temporary(ConstraintVector), $
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               H2O_HDO_Corr_QA : temporary(H2O_HDO_Corr_QA), $
               InformationContent : temporary(InformationContent), $
               Initial : temporary(Initial), $    
               KDotDL_QA : temporary(KDotDL_QA), $    
               LDotDL_QA : temporary(LDotDL_QA), $    
               MaxNumIterations : temporary(MaxNumIterations), $
               MeasurementErrorCovariance : temporary(MeasurementErrorCovariance), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               ObservationErrorCovariance : temporary(ObservationErrorCovariance), $
               O3_Ccurve_QA : temporary(O3_Ccurve_QA), $
; FIX
;               OzoneIRFK : temporary(OzoneIRFK), $
               OzoneTroposphericColumn : temporary(OzoneTroposphericColumn), $
               OzoneTroposphericColumnError : temporary(OzoneTroposphericColumnError), $
               OzoneTroposphericColumnInitial : temporary(OzoneTroposphericColumnInitial), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               Scan_Averaged_Count : temporary(Scan_Averaged_Count), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SpeciesRetrievalQuality : temporary(SpeciesRetrievalQuality), $
               SurfaceEmissMean_QA : temporary(SurfaceEmissMean_QA), $    
               SurfaceEmissionLayer_QA : temporary(SurfaceEmissionLayer_QA), $    
               SurfaceTempConstraint : temporary(SurfaceTempConstraint), $    
               SurfaceTempDegreesOfFreedom : temporary(SurfaceTempDegreesOfFreedom), $    
               SurfaceTempError : temporary(SurfaceTempError), $
               SurfaceTempInitial : temporary(SurfaceTempInitial), $    
               SurfaceTempObservationError : temporary(SurfaceTempObservationError), $    
               SurfaceTempPrecision : temporary(SurfaceTempPrecision), $    
               SurfaceTempVsApriori_QA : temporary(SurfaceTempVsApriori_QA), $    
               SurfaceTempVsAtmTemp_QA : temporary(SurfaceTempVsAtmTemp_QA), $    
               SurfaceTemperature : temporary(SurfaceTemperature), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalColumnDensityInitial : temporary(TotalColumnDensityInitial), $    
               TotalError : temporary(TotalError), $
               TotalErrorCovariance : temporary(TotalErrorCovariance), $
               TropopausePressure : temporary(TropopausePressure), $
               UTCTime : temporary(UTCTime), $
               CloudFrequency : temporary(CloudFrequency),$
               EmissivityWavenumber : temporary(EmissivityWavenumber), $
               pressuregrid : temporary(pressuregrid), $
               VerticalCoordinate : temporary(VerticalCoordinate), $
               Scan_Resolution : temporary(Scan_Resolution) $  
               }
endif 
if version eq 6 and type eq 'Limb' then begin
    species_data = { $
               species : temporary(species), $
               Precision : temporary(Precision), $
               VerticalResolution : temporary(VerticalResolution), $
               AirDensity : temporary(AirDensity), $    
               Altitude : temporary(Altitude), $
               AverageCloudEffOpticalDepth : temporary(AverageCloudEffOpticalDepth), $    
               AverageCloudEffOpticalDepthError : temporary(AverageCloudEffOpticalDepthError), $    
               AveragingKernel : temporary(Averagingkernel), $
               AveragingKernelDiagonal : temporary(AveragingKernelDiagonal), $    
               Calibration_QA : temporary(Calibration_QA), $
               CloudEffectiveOpticalDepth : temporary(CloudEffectiveOpticalDepth), $
               CloudEffectiveOpticalDepthError : temporary(CloudEffectiveOpticalDepthError), $
               CloudTopPressure : temporary(CloudTopPressure), $
               CloudTopPressureError : temporary(CloudTopPressureError), $
               CloudVariability_QA : temporary(CloudVariability_QA), $    
               ConstraintVector : temporary(ConstraintVector), $
               DegreesOfFreedomForSignal : temporary(DegreesOfFreedomForSignal), $
               DeviationVsRetrievalCovariance : $
               		temporary(DeviationVsRetrievalCovariance), $
               H2O_HDO_Corr_QA : temporary(H2O_HDO_Corr_QA), $
               InformationContent : temporary(InformationContent), $
               Initial : temporary(Initial), $    
               KDotDL_QA : temporary(KDotDL_QA), $    
               LDotDL_QA : temporary(LDotDL_QA), $    
               MaxNumIterations : temporary(MaxNumIterations), $
               MeasurementErrorCovariance : temporary(MeasurementErrorCovariance), $
               NumberIterPerformed : temporary(NumberIterPerformed), $
               ObservationErrorCovariance : temporary(ObservationErrorCovariance), $
               Pressure : temporary(Pressure), $
               RadianceResidualMax : temporary(RadianceResidualMax), $
               RadianceResidualMean : temporary(RadianceResidualMean), $
               RadianceResidualRMS : temporary(RadianceResidualRMS), $
               SpeciesRetrievalConverged : temporary(SpeciesRetrievalConverged), $
               SpeciesRetrievalQuality : temporary(SpeciesRetrievalQuality), $
               TotalColumnDensity : temporary(TotalColumnDensity), $
               TotalColumnDensityError : temporary(TotalColumnDensityError), $
               TotalColumnDensityInitial : temporary(TotalColumnDensityInitial), $    
               TotalError : temporary(TotalError), $
               TotalErrorCovariance : temporary(TotalErrorCovariance), $
               TropopausePressure : temporary(TropopausePressure), $
               UTCTime : temporary(UTCTime), $
               CloudFrequency : temporary(CloudFrequency),$
               EmissivityWavenumber : temporary(EmissivityWavenumber), $
               pressuregrid : temporary(pressuregrid), $
               VerticalCoordinate : temporary(VerticalCoordinate), $
               Scan_Resolution : temporary(Scan_Resolution) $  
               }
endif 

;
; put all attribute data into return structure
;
if version eq 1 then begin
    species_attributes = { $
                     species : temporary(attrib0), $
                     Precision : temporary(attrib1), $
                     VerticalResolution : temporary(attrib2), $
                     Altitude : temporary(attrib4), $
                     AverageingKernel : temporary(attrib6), $
                     CloudTopHeight : temporary(attrib11), $
                     CloudTopPressure : temporary(attrib12), $
                     ConstraintVector : temporary(attrib15), $
                     DegreesOfFreedomForSignal : temporary(attrib16), $
                     DeviationVsRetrievalCovariance : temporary(attrib17),$
                     InformationContent : temporary(attrib18), $
                     MaxNumIterations : temporary(attrib22), $
                     MeasurementErrorCovariance : temporary(attrib23), $
                     NumbIterPerformed : temporary(attrib24), $
                     Pressure : temporary(attrib26), $
                     RadianceResidualMax : temporary(attrib27), $
                     RadianceResidualMean : temporary(attrib28), $
                     RadianceResidualRMS : temporary(attrib29), $
                     Scan_Averaged_Count : temporary(attrib30), $
                     SpeciesRetrievalConverged : temporary(attrib31), $
                     SurfaceTempError : temporary(attrib36), $
                     SurfaceTemperature : temporary(attrib42), $
                     SystematicErrorCovariance : temporary(attrib43), $
                     TotalColumnDensity : temporary(attrib44), $
                     TotalColumnDensityError : temporary(attrib45), $
                     TotalError : temporary(attrib47), $
                     TotalErrorCovariance : temporary(attrib48) $
                     }
endif 
if version eq 2 then begin
    species_attributes = { $
                     species : temporary(attrib0), $
                     Precision : temporary(attrib1), $
                     VerticalResolution : temporary(attrib2), $
                     Altitude : temporary(attrib4), $
                     AverageingKernel : temporary(attrib6), $
                     CloudEffectiveOpticalDepth : temporary(attrib9), $
    		     CloudEffectiveOpticalDepthError : temporary(attrib10), $
                     CloudTopPressure : temporary(attrib12), $
                     CloudTopPressureError : temporary(attrib13), $
                     ConstraintVector : temporary(attrib15), $
                     DegreesOfFreedomForSignal : temporary(attrib16), $
                     DeviationVsRetrievalCovariance : temporary(attrib17),$
                     InformationContent : temporary(attrib18), $
                     MaxNumIterations : temporary(attrib22), $
                     MeasurementErrorCovariance : temporary(attrib23), $
                     NumbIterPerformed : temporary(attrib24), $
                     Pressure : temporary(attrib26), $
                     RadianceResidualMax : temporary(attrib27), $
                     RadianceResidualMean : temporary(attrib28), $
                     RadianceResidualRMS : temporary(attrib29), $
                     Scan_Averaged_Count : temporary(attrib30), $
                     SpeciesRetrievalConverged : temporary(attrib31), $
                     SpeciesRetrievalQuality : temporary(attrib32), $
                     SurfaceTempError : temporary(attrib36), $
                     SurfaceTemperature : temporary(attrib42), $
                     SystematicErrorCovariance : temporary(attrib43), $
                     TotalColumnDensity : temporary(attrib44), $
                     TotalColumnDensityError : temporary(attrib45), $
                     TotalError : temporary(attrib47), $
                     TotalErrorCovariance : temporary(attrib48) $
                     }
endif 




if version eq 3 and type eq 'Nadir' then begin
    species_attributes = { $
                     species : temporary(attrib0), $
                     Precision : temporary(attrib1), $
                     VerticalResolution : temporary(attrib2), $
                     AirDensity : temporary(attrib3), $
                     Altitude : temporary(attrib4), $
                     AverageCloudEffOpticalDepth : temporary(attrib5), $
                     AverageingKernel : temporary(attrib6), $
                     AveragingKernelDiagonal : temporary(attrib7), $
                     Calibration_QA : temporary(attrib8), $
                     CloudEffectiveOpticalDepth : temporary(attrib9), $
    		     CloudEffectiveOpticalDepthError : temporary(attrib10), $
                     CloudTopPressure : temporary(attrib12), $
                     CloudTopPressureError : temporary(attrib13), $
                     CloudVariability_QA : temporary(attrib14), $
                     ConstraintVector : temporary(attrib15), $
                     DegreesOfFreedomForSignal : temporary(attrib16), $
                     DeviationVsRetrievalCovariance : temporary(attrib17),$
                     InformationContent : temporary(attrib18), $
                     Inital : temporary(attrib19), $
                     KDotDL_QA : temporary(attrib20), $
                     LDotDL_QA : temporary(attrib21), $
                     MaxNumIterations : temporary(attrib22), $
                     MeasurementErrorCovariance : temporary(attrib23), $
                     NumbIterPerformed : temporary(attrib24), $
                     ObservationErrorCovariance : temporary(attrib25), $
                     Pressure : temporary(attrib26), $
                     RadianceResidualMax : temporary(attrib27), $
                     RadianceResidualMean : temporary(attrib28), $
                     RadianceResidualRMS : temporary(attrib29), $
                     Scan_Averaged_Count : temporary(attrib30), $
                     SpeciesRetrievalConverged : temporary(attrib31), $
                     SpeciesRetrievalQuality : temporary(attrib32), $
                     SurfaceEmissMean_QA : temporary(attrib33), $
                     SurfaceTempConstraint : temporary(attrib34), $
                     SurfaceTempDegreesOfFreedom : temporary(attrib35), $
                     SurfaceTempError : temporary(attrib36), $
                     SurfaceTempInitial : temporary(attrib37), $
                     SurfaceTempObservationError : temporary(attrib38), $
                     SurfaceTempPrecision : temporary(attrib39), $
                     SurfaceTempVsApriori_QA : temporary(attrib40), $
                     SurfaceTempVsAtmTemp_QA : temporary(attrib41), $
                     SurfaceTemperature : temporary(attrib42), $
                     TotalColumnDensity : temporary(attrib44), $
                     TotalColumnDensityError : temporary(attrib45), $
                     TotalColumnDensityInitial : temporary(attrib46), $
                     TotalError : temporary(attrib47), $
                     TotalErrorCovariance : temporary(attrib48) $
                     }
endif 

if version eq 3 and type eq 'Limb' then begin
    species_attributes = { $
                     species : temporary(attrib0), $
                     Precision : temporary(attrib1), $
                     VerticalResolution : temporary(attrib2), $
                     AirDensity : temporary(attrib3), $
                     Altitude : temporary(attrib4), $
                     AverageCloudEffOpticalDepth : temporary(attrib5), $
                     AverageingKernel : temporary(attrib6), $
                     AveragingKernelDiagonal : temporary(attrib7), $
                     Calibration_QA : temporary(attrib8), $
                     CloudEffectiveOpticalDepth : temporary(attrib9), $
    		     CloudEffectiveOpticalDepthError : temporary(attrib10), $
                     CloudTopPressure : temporary(attrib12), $
                     CloudTopPressureError : temporary(attrib13), $
                     CloudVariability_QA : temporary(attrib14), $
                     ConstraintVector : temporary(attrib15), $
                     DegreesOfFreedomForSignal : temporary(attrib16), $
                     DeviationVsRetrievalCovariance : temporary(attrib17),$
                     InformationContent : temporary(attrib18), $
                     Inital : temporary(attrib19), $
                     KDotDL_QA : temporary(attrib20), $
                     LDotDL_QA : temporary(attrib21), $
                     MaxNumIterations : temporary(attrib22), $
                     MeasurementErrorCovariance : temporary(attrib23), $
                     NumbIterPerformed : temporary(attrib24), $
                     ObservationErrorCovariance : temporary(attrib25), $
                     Pressure : temporary(attrib26), $
                     RadianceResidualMax : temporary(attrib27), $
                     RadianceResidualMean : temporary(attrib28), $
                     RadianceResidualRMS : temporary(attrib29), $
                     SpeciesRetrievalConverged : temporary(attrib31), $
                     SpeciesRetrievalQuality : temporary(attrib32), $
                     TotalColumnDensity : temporary(attrib44), $
                     TotalColumnDensityError : temporary(attrib45), $
                     TotalColumnDensityInitial : temporary(attrib46), $
                     TotalError : temporary(attrib47), $
                     TotalErrorCovariance : temporary(attrib48) $
                     }
endif 

if version eq 4 and type eq 'Nadir' then begin
    species_attributes = { $
                     species : temporary(attrib0), $
                     Precision : temporary(attrib1), $
                     VerticalResolution : temporary(attrib2), $
                     AirDensity : temporary(attrib3), $
                     Altitude : temporary(attrib4), $
                     AverageCloudEffOpticalDepth : temporary(attrib5), $
                     AverageingKernel : temporary(attrib6), $
                     AveragingKernelDiagonal : temporary(attrib7), $
                     Calibration_QA : temporary(attrib8), $
                     CloudEffectiveOpticalDepth : temporary(attrib9), $
    		     CloudEffectiveOpticalDepthError : temporary(attrib10), $
                     CloudTopPressure : temporary(attrib12), $
                     CloudTopPressureError : temporary(attrib13), $
                     CloudVariability_QA : temporary(attrib14), $
                     ConstraintVector : temporary(attrib15), $
                     DegreesOfFreedomForSignal : temporary(attrib16), $
                     DeviationVsRetrievalCovariance : temporary(attrib17),$
                     H2O_HDO_Corr_QA : temporary(attrib51), $
                     InformationContent : temporary(attrib18), $
                     Inital : temporary(attrib19), $
                     KDotDL_QA : temporary(attrib20), $
                     LDotDL_QA : temporary(attrib21), $
                     MaxNumIterations : temporary(attrib22), $
                     MeasurementErrorCovariance : temporary(attrib23), $
                     NumbIterPerformed : temporary(attrib24), $
                     ObservationErrorCovariance : temporary(attrib25), $
                     Pressure : temporary(attrib26), $
                     RadianceResidualMax : temporary(attrib27), $
                     RadianceResidualMean : temporary(attrib28), $
                     RadianceResidualRMS : temporary(attrib29), $
                     Scan_Averaged_Count : temporary(attrib30), $
                     SpeciesRetrievalConverged : temporary(attrib31), $
                     SpeciesRetrievalQuality : temporary(attrib32), $
                     SurfaceEmissMean_QA : temporary(attrib33), $
                     SurfaceEmissionLayer_QA : temporary(attrib50), $
                     SurfaceTempConstraint : temporary(attrib34), $
                     SurfaceTempDegreesOfFreedom : temporary(attrib35), $
                     SurfaceTempError : temporary(attrib36), $
                     SurfaceTempInitial : temporary(attrib37), $
                     SurfaceTempObservationError : temporary(attrib38), $
                     SurfaceTempPrecision : temporary(attrib39), $
                     SurfaceTempVsApriori_QA : temporary(attrib40), $
                     SurfaceTempVsAtmTemp_QA : temporary(attrib41), $
                     SurfaceTemperature : temporary(attrib42), $
                     TotalColumnDensity : temporary(attrib44), $
                     TotalColumnDensityError : temporary(attrib45), $
                     TotalColumnDensityInitial : temporary(attrib46), $
                     TotalError : temporary(attrib47), $
                     TotalErrorCovariance : temporary(attrib48), $
                     UTCTime : temporary(attrib49) $
                     }
endif 

if version eq 4 and type eq 'Limb' then begin
    species_attributes = { $
                     species : temporary(attrib0), $
                     Precision : temporary(attrib1), $
                     VerticalResolution : temporary(attrib2), $
                     AirDensity : temporary(attrib3), $
                     Altitude : temporary(attrib4), $
                     AverageCloudEffOpticalDepth : temporary(attrib5), $
                     AverageingKernel : temporary(attrib6), $
                     AveragingKernelDiagonal : temporary(attrib7), $
                     Calibration_QA : temporary(attrib8), $
                     CloudEffectiveOpticalDepth : temporary(attrib9), $
    		     CloudEffectiveOpticalDepthError : temporary(attrib10), $
                     CloudTopPressure : temporary(attrib12), $
                     CloudTopPressureError : temporary(attrib13), $
                     CloudVariability_QA : temporary(attrib14), $
                     ConstraintVector : temporary(attrib15), $
                     DegreesOfFreedomForSignal : temporary(attrib16), $
                     DeviationVsRetrievalCovariance : temporary(attrib17),$
                     H2O_HDO_Corr_QA : temporary(attrib51), $
                     InformationContent : temporary(attrib18), $
                     Inital : temporary(attrib19), $
                     KDotDL_QA : temporary(attrib20), $
                     LDotDL_QA : temporary(attrib21), $
                     MaxNumIterations : temporary(attrib22), $
                     MeasurementErrorCovariance : temporary(attrib23), $
                     NumbIterPerformed : temporary(attrib24), $
                     ObservationErrorCovariance : temporary(attrib25), $
                     Pressure : temporary(attrib26), $
                     RadianceResidualMax : temporary(attrib27), $
                     RadianceResidualMean : temporary(attrib28), $
                     RadianceResidualRMS : temporary(attrib29), $
                     SpeciesRetrievalConverged : temporary(attrib31), $
                     SpeciesRetrievalQuality : temporary(attrib32), $
                     TotalColumnDensity : temporary(attrib44), $
                     TotalColumnDensityError : temporary(attrib45), $
                     TotalColumnDensityInitial : temporary(attrib46), $
                     TotalError : temporary(attrib47), $
                     TotalErrorCovariance : temporary(attrib48), $
                     UTCTime : temporary(attrib49) $
                     }
endif 

if version eq 5 and type eq 'Nadir' then begin
    species_attributes = { $
                     species : temporary(attrib0), $
                     Precision : temporary(attrib1), $
                     VerticalResolution : temporary(attrib2), $
                     AirDensity : temporary(attrib3), $
                     Altitude : temporary(attrib4), $
                     AverageCloudEffOpticalDepth : temporary(attrib5), $
                     AverageCloudEffOpticalDepthError : temporary(attrib5a), $
                     AverageingKernel : temporary(attrib6), $
                     AveragingKernelDiagonal : temporary(attrib7), $
                     Calibration_QA : temporary(attrib8), $
                     CloudEffectiveOpticalDepth : temporary(attrib9), $
    		     CloudEffectiveOpticalDepthError : temporary(attrib10), $
                     CloudTopPressure : temporary(attrib12), $
                     CloudTopPressureError : temporary(attrib13), $
                     CloudVariability_QA : temporary(attrib14), $
                     ConstraintVector : temporary(attrib15), $
                     DegreesOfFreedomForSignal : temporary(attrib16), $
                     DeviationVsRetrievalCovariance : temporary(attrib17),$
                     H2O_HDO_Corr_QA : temporary(attrib51), $
                     InformationContent : temporary(attrib18), $
                     Inital : temporary(attrib19), $
                     KDotDL_QA : temporary(attrib20), $
                     LDotDL_QA : temporary(attrib21), $
                     MaxNumIterations : temporary(attrib22), $
                     MeasurementErrorCovariance : temporary(attrib23), $
                     NumbIterPerformed : temporary(attrib24), $
                     ObservationErrorCovariance : temporary(attrib25), $
                     O3_Ccurve_QA : temporary(attrib26a), $
                     Pressure : temporary(attrib26), $
                     RadianceResidualMax : temporary(attrib27), $
                     RadianceResidualMean : temporary(attrib28), $
                     RadianceResidualRMS : temporary(attrib29), $
                     Scan_Averaged_Count : temporary(attrib30), $
                     SpeciesRetrievalConverged : temporary(attrib31), $
                     SpeciesRetrievalQuality : temporary(attrib32), $
                     SurfaceEmissMean_QA : temporary(attrib33), $
                     SurfaceEmissionLayer_QA : temporary(attrib50), $
                     SurfaceTempConstraint : temporary(attrib34), $
                     SurfaceTempDegreesOfFreedom : temporary(attrib35), $
                     SurfaceTempError : temporary(attrib36), $
                     SurfaceTempInitial : temporary(attrib37), $
                     SurfaceTempObservationError : temporary(attrib38), $
                     SurfaceTempPrecision : temporary(attrib39), $
                     SurfaceTempVsApriori_QA : temporary(attrib40), $
                     SurfaceTempVsAtmTemp_QA : temporary(attrib41), $
                     SurfaceTemperature : temporary(attrib42), $
                     TotalColumnDensity : temporary(attrib44), $
                     TotalColumnDensityError : temporary(attrib45), $
                     TotalColumnDensityInitial : temporary(attrib46), $
                     TotalError : temporary(attrib47), $
                     TotalErrorCovariance : temporary(attrib48), $
                     UTCTime : temporary(attrib49) $
                     }
endif 

if version eq 5 and type eq 'Limb' then begin
    species_attributes = { $
                     species : temporary(attrib0), $
                     Precision : temporary(attrib1), $
                     VerticalResolution : temporary(attrib2), $
                     AirDensity : temporary(attrib3), $
                     Altitude : temporary(attrib4), $
                     AverageCloudEffOpticalDepth : temporary(attrib5), $
                     AverageCloudEffOpticalDepthErr : temporary(attrib5a), $
                     AverageingKernel : temporary(attrib6), $
                     AveragingKernelDiagonal : temporary(attrib7), $
                     Calibration_QA : temporary(attrib8), $
                     CloudEffectiveOpticalDepth : temporary(attrib9), $
    		     CloudEffectiveOpticalDepthError : temporary(attrib10), $
                     CloudTopPressure : temporary(attrib12), $
                     CloudTopPressureError : temporary(attrib13), $
                     CloudVariability_QA : temporary(attrib14), $
                     ConstraintVector : temporary(attrib15), $
                     DegreesOfFreedomForSignal : temporary(attrib16), $
                     DeviationVsRetrievalCovariance : temporary(attrib17),$
                     H2O_HDO_Corr_QA : temporary(attrib51), $
                     InformationContent : temporary(attrib18), $
                     Inital : temporary(attrib19), $
                     KDotDL_QA : temporary(attrib20), $
                     LDotDL_QA : temporary(attrib21), $
                     MaxNumIterations : temporary(attrib22), $
                     MeasurementErrorCovariance : temporary(attrib23), $
                     NumbIterPerformed : temporary(attrib24), $
                     ObservationErrorCovariance : temporary(attrib25), $
                     Pressure : temporary(attrib26), $
                     RadianceResidualMax : temporary(attrib27), $
                     RadianceResidualMean : temporary(attrib28), $
                     RadianceResidualRMS : temporary(attrib29), $
                     SpeciesRetrievalConverged : temporary(attrib31), $
                     SpeciesRetrievalQuality : temporary(attrib32), $
                     TotalColumnDensity : temporary(attrib44), $
                     TotalColumnDensityError : temporary(attrib45), $
                     TotalColumnDensityInitial : temporary(attrib46), $
                     TotalError : temporary(attrib47), $
                     TotalErrorCovariance : temporary(attrib48), $
                     UTCTime : temporary(attrib49) $
                     }
endif 

if version eq 6 and type eq 'Nadir' then begin
    species_attributes = { $
                     species : temporary(attrib0), $
                     Precision : temporary(attrib1), $
                     VerticalResolution : temporary(attrib2), $
                     AirDensity : temporary(attrib3), $
                     Altitude : temporary(attrib4), $
                     AverageCloudEffOpticalDepth : temporary(attrib5), $
                     AverageCloudEffOpticalDepthError : temporary(attrib5a), $
                     AverageingKernel : temporary(attrib6), $
                     AveragingKernelDiagonal : temporary(attrib7), $
                     Calibration_QA : temporary(attrib8), $
                     CloudEffectiveOpticalDepth : temporary(attrib9), $
    		     CloudEffectiveOpticalDepthError : temporary(attrib10), $
                     CloudTopPressure : temporary(attrib12), $
                     CloudTopPressureError : temporary(attrib13), $
                     CloudVariability_QA : temporary(attrib14), $
                     ConstraintVector : temporary(attrib15), $
                     DegreesOfFreedomForSignal : temporary(attrib16), $
                     DeviationVsRetrievalCovariance : temporary(attrib17),$
                     H2O_HDO_Corr_QA : temporary(attrib51), $
                     InformationContent : temporary(attrib18), $
                     Inital : temporary(attrib19), $
                     KDotDL_QA : temporary(attrib20), $
                     LDotDL_QA : temporary(attrib21), $
                     MaxNumIterations : temporary(attrib22), $
                     MeasurementErrorCovariance : temporary(attrib23), $
                     NumbIterPerformed : temporary(attrib24), $
                     ObservationErrorCovariance : temporary(attrib25), $
                     O3_Ccurve_QA : temporary(attrib26a), $
; FIX
;               OzoneIRFK : temporary(attrib26e), $
               OzoneTroposphericColumn : temporary(attrib26b), $
               OzoneTroposphericColumnError : temporary(attrib26c), $
               OzoneTroposphericColumnInitial : temporary(attrib26d), $
                     Pressure : temporary(attrib26), $
                     RadianceResidualMax : temporary(attrib27), $
                     RadianceResidualMean : temporary(attrib28), $
                     RadianceResidualRMS : temporary(attrib29), $
                     Scan_Averaged_Count : temporary(attrib30), $
                     SpeciesRetrievalConverged : temporary(attrib31), $
                     SpeciesRetrievalQuality : temporary(attrib32), $
                     SurfaceEmissMean_QA : temporary(attrib33), $
                     SurfaceEmissionLayer_QA : temporary(attrib50), $
                     SurfaceTempConstraint : temporary(attrib34), $
                     SurfaceTempDegreesOfFreedom : temporary(attrib35), $
                     SurfaceTempError : temporary(attrib36), $
                     SurfaceTempInitial : temporary(attrib37), $
                     SurfaceTempObservationError : temporary(attrib38), $
                     SurfaceTempPrecision : temporary(attrib39), $
                     SurfaceTempVsApriori_QA : temporary(attrib40), $
                     SurfaceTempVsAtmTemp_QA : temporary(attrib41), $
                     SurfaceTemperature : temporary(attrib42), $
                     TotalColumnDensity : temporary(attrib44), $
                     TotalColumnDensityError : temporary(attrib45), $
                     TotalColumnDensityInitial : temporary(attrib46), $
                     TotalError : temporary(attrib47), $
                     TotalErrorCovariance : temporary(attrib48), $
               TropopausePressure : temporary(attrib48a), $
                     UTCTime : temporary(attrib49) $
                     }
endif 

if version eq 6 and type eq 'Limb' then begin
    species_attributes = { $
                     species : temporary(attrib0), $
                     Precision : temporary(attrib1), $
                     VerticalResolution : temporary(attrib2), $
                     AirDensity : temporary(attrib3), $
                     Altitude : temporary(attrib4), $
                     AverageCloudEffOpticalDepth : temporary(attrib5), $
                     AverageCloudEffOpticalDepthErr : temporary(attrib5a), $
                     AverageingKernel : temporary(attrib6), $
                     AveragingKernelDiagonal : temporary(attrib7), $
                     Calibration_QA : temporary(attrib8), $
                     CloudEffectiveOpticalDepth : temporary(attrib9), $
    		     CloudEffectiveOpticalDepthError : temporary(attrib10), $
                     CloudTopPressure : temporary(attrib12), $
                     CloudTopPressureError : temporary(attrib13), $
                     CloudVariability_QA : temporary(attrib14), $
                     ConstraintVector : temporary(attrib15), $
                     DegreesOfFreedomForSignal : temporary(attrib16), $
                     DeviationVsRetrievalCovariance : temporary(attrib17),$
                     H2O_HDO_Corr_QA : temporary(attrib51), $
                     InformationContent : temporary(attrib18), $
                     Inital : temporary(attrib19), $
                     KDotDL_QA : temporary(attrib20), $
                     LDotDL_QA : temporary(attrib21), $
                     MaxNumIterations : temporary(attrib22), $
                     MeasurementErrorCovariance : temporary(attrib23), $
                     NumbIterPerformed : temporary(attrib24), $
                     ObservationErrorCovariance : temporary(attrib25), $
                     Pressure : temporary(attrib26), $
                     RadianceResidualMax : temporary(attrib27), $
                     RadianceResidualMean : temporary(attrib28), $
                     RadianceResidualRMS : temporary(attrib29), $
                     SpeciesRetrievalConverged : temporary(attrib31), $
                     SpeciesRetrievalQuality : temporary(attrib32), $
                     TotalColumnDensity : temporary(attrib44), $
                     TotalColumnDensityError : temporary(attrib45), $
                     TotalColumnDensityInitial : temporary(attrib46), $
                     TotalError : temporary(attrib47), $
                     TotalErrorCovariance : temporary(attrib48), $
               TropopausePressure : temporary(attrib48a), $
                     UTCTime : temporary(attrib49) $
                     }
endif 


;
; get geolocation data
;
geoid = h5g_open(specid,'Geolocation Fields')
;
;
dataid = h5d_open(geoid,'BoresightAzimuth')
BoresightAzimuth = h5d_read(dataid)
get_data_attributes,dataid,size(BoresightAzimuth,/type),attrib0
h5d_close,dataid

dataid = h5d_open(geoid,'BoresightNadirAngle')
BoresightNadirAngle = h5d_read(dataid)
get_data_attributes,dataid,size(BoresightNadirAngle,/type),attrib1
h5d_close,dataid

dataid = h5d_open(geoid,'BoresightNadirAngleUnc')
BoresightNadirAngleUnc = h5d_read(dataid)
get_data_attributes,dataid,size(BoresightNadirAngleUnc,/type),attrib2
h5d_close,dataid

if version ge 3 and type eq 'Limb' then begin
    dataid = h5d_open(geoid,'BoresightTangentHeight')
    BoresightTangentHeight = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightTangentHeight,/type),attrib3
    h5d_close,dataid

    dataid = h5d_open(geoid,'BoresightTangentHeightUnc')
    BoresightTangentHeightUnc = h5d_read(dataid)
    get_data_attributes,dataid,size(BoresightTangentHeightUnc,/type),attrib4
    h5d_close,dataid
endif

if version ge 3 then begin
    dataid = h5d_open(geoid,'DayNightFlag')
    DayNightFlag = h5d_read(dataid)
    get_data_attributes,dataid,size(DayNightFlag,/type),attrib5
    h5d_close,dataid
endif

if version ge 3 and type eq 'Nadir' then begin
    dataid = h5d_open(geoid,'DominantSurfaceType')
    DominantSurfaceType = h5d_read(dataid)
    get_data_attributes,dataid,size(DominantSurfaceType,/type),attrib6
    h5d_close,dataid
endif


dataid = h5d_open(geoid,'Latitude')
Latitude = h5d_read(dataid)
get_data_attributes,dataid,size(Latitude,/type),attrib7
h5d_close,dataid

dataid = h5d_open(geoid,'Latitude_Footprint_1')
Latitude_Footprint_1 = h5d_read(dataid)
get_data_attributes,dataid,size(Latitude_Footprint_1,/type),attrib8
h5d_close,dataid

dataid = h5d_open(geoid,'Latitude_Footprint_2')
Latitude_Footprint_2 = h5d_read(dataid)
get_data_attributes,dataid,size(Latitude_Footprint_2,/type),attrib9
h5d_close,dataid

dataid = h5d_open(geoid,'Latitude_Footprint_3')
Latitude_Footprint_3 = h5d_read(dataid)
get_data_attributes,dataid,size(Latitude_Footprint_3,/type),attrib10
h5d_close,dataid

dataid = h5d_open(geoid,'Latitude_Footprint_4')
Latitude_Footprint_4 = h5d_read(dataid)
get_data_attributes,dataid,size(Latitude_Footprint_4,/type),attrib11
h5d_close,dataid

dataid = h5d_open(geoid,'LocalSolarTime')
LocalSolarTime = h5d_read(dataid)
get_data_attributes,dataid,size(LocalSolarTime,/type),attrib12
h5d_close,dataid

dataid = h5d_open(geoid,'Longitude')
Longitude = h5d_read(dataid)
get_data_attributes,dataid,size(Longitude,/type),attrib13
h5d_close,dataid

dataid = h5d_open(geoid,'Longitude_Footprint_1')
Longitude_Footprint_1 = h5d_read(dataid)
get_data_attributes,dataid,size(Longitude_Footprint_1,/type),attrib14
h5d_close,dataid

dataid = h5d_open(geoid,'Longitude_Footprint_2')
Longitude_Footprint_2 = h5d_read(dataid)
get_data_attributes,dataid,size(Longitude_Footprint_2,/type),attrib15
h5d_close,dataid

dataid = h5d_open(geoid,'Longitude_Footprint_3')
Longitude_Footprint_3 = h5d_read(dataid)
get_data_attributes,dataid,size(Longitude_Footprint_3,/type),attrib16
h5d_close,dataid

dataid = h5d_open(geoid,'Longitude_Footprint_4')
Longitude_Footprint_4 = h5d_read(dataid)
get_data_attributes,dataid,size(Longitude_Footprint_4,/type),attrib17
h5d_close,dataid

if version ge 2 then begin
    dataid = h5d_open(geoid,'Scan')
    Scan = h5d_read(dataid)
    get_data_attributes,dataid,size(Scan,/type),attrib18
    h5d_close,dataid
    
    dataid = h5d_open(geoid,'Sequence')
    Sequence = h5d_read(dataid)
    get_data_attributes,dataid,size(Sequence,/type),attrib19
    h5d_close,dataid
endif

dataid = h5d_open(geoid,'SolarZenithAngle')
SolarZenithAngle = h5d_read(dataid)
get_data_attributes,dataid,size(SolarZenithAngle,/type),attrib20
h5d_close,dataid

dataid = h5d_open(geoid,'SurfaceElevStandardDeviation')
SurfaceElevStandardDeviation = h5d_read(dataid)
get_data_attributes,dataid,size(SurfaceElevStandardDeviation,/type),attrib21
h5d_close,dataid

if version ge 3 then begin
    dataid = h5d_open(geoid,'SurfaceTypeFootprint')
    SurfaceTypeFootprint = h5d_read(dataid)
    get_data_attributes,dataid,size(SurfaceTypeFootprint,/type),attrib22
    h5d_close,dataid
endif


dataid = h5d_open(geoid,'Tgt_SpacecraftAzimuth')
Tgt_SpacecraftAzimuth = h5d_read(dataid)
get_data_attributes,dataid,size(Tgt_SpacecraftAzimuth,/type),attrib23
h5d_close,dataid

dataid = h5d_open(geoid,'Tgt_SpacecraftZenith')
Tgt_SpacecraftZenith = h5d_read(dataid)
get_data_attributes,dataid,size(Tgt_SpacecraftZenith,/type),attrib24
h5d_close,dataid

dataid = h5d_open(geoid,'Time')
Time = h5d_read(dataid)
get_data_attributes,dataid,size(Time,/type),attrib25
h5d_close,dataid

if version eq 3 then begin
    dataid = h5d_open(geoid,'UTCTime')
    ua = h5d_read(dataid)
    get_data_attributes,dataid,size(ua,/type),attrib26,/special
    h5d_close,dataid
    ss = size(ua)
    UTCTime = strarr(ss[2])
;    help,ua
    for i=0l,ss[2]-1 do begin
        utctime[i] = ua[0,i] +ua[1,i] +ua[2,i] +ua[3,i] +ua[4,i] +ua[5,i] + $
                     ua[6,i] +ua[7,i] +ua[8,i] +ua[9,i] +ua[10,i]+ua[11,i]+ $
                     ua[12,i]+ua[13,i]+ua[14,i]+ua[15,i]+ua[16,i]+ua[17,i]+ $
                     ua[18,i]+ua[19,i]+ua[20,i]+ua[21,i]+ua[22,i]+ua[23,i]+ $
                     ua[24,i]+ua[25,i]+ua[26,i]
    endfor
endif

; close tags
h5g_close,geoid
h5g_close,specid
h5g_close,swid
;
; put data into return structure
;
if version eq 1 then begin
    geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
           Latitude : temporary(Latitude), $
           Latitude_Footprint_1 : temporary(Latitude_Footprint_1), $
           Latitude_Footprint_2 : temporary(Latitude_Footprint_2), $
           Latitude_Footprint_3 : temporary(Latitude_Footprint_3), $
           Latitude_Footprint_4 : temporary(Latitude_Footprint_4), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Longitude_Footprint_1 : temporary(Longitude_Footprint_1), $
           Longitude_Footprint_2 : temporary(Longitude_Footprint_2), $
           Longitude_Footprint_3 : temporary(Longitude_Footprint_3), $
           Longitude_Footprint_4 : temporary(Longitude_Footprint_4), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time) }

    geo_attributes = { $
                 BoresightAzimuth : temporary(attrib0), $
                 BoresightNadirAngle : temporary(attrib1), $
                 BoresightNadirAngleUnc : temporary(attrib2), $
                 Latitude : temporary(attrib7), $
                 Latitude_Footprint_1 : temporary(attrib8), $
                 Latitude_Footprint_2 : temporary(attrib9), $
                 Latitude_Footprint_3 : temporary(attrib10), $
                 Latitude_Footprint_4 : temporary(attrib11), $
                 LocalSolarTime : temporary(attrib12), $
                 Longitude : temporary(attrib13), $
                 Longitude_Footprint_1 : temporary(attrib14), $
                 Longitude_Footprint_2 : temporary(attrib15), $
                 Longitude_Footprint_3 : temporary(attrib16), $
                 Longitude_Footprint_4 : temporary(attrib17), $
                 SolarZenithAngle : temporary(attrib20), $
                 SurfaceElevStandardDeviation :  temporary(attrib21), $
                 Tgt_SpacecraftAzimuth : temporary(attrib23), $
                 Tgt_SpacecraftZenith : temporary(attrib24), $
                 Time : temporary(attrib25) }
endif
if version eq 2 then begin
    geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
           Latitude : temporary(Latitude), $
           Latitude_Footprint_1 : temporary(Latitude_Footprint_1), $
           Latitude_Footprint_2 : temporary(Latitude_Footprint_2), $
           Latitude_Footprint_3 : temporary(Latitude_Footprint_3), $
           Latitude_Footprint_4 : temporary(Latitude_Footprint_4), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Longitude_Footprint_1 : temporary(Longitude_Footprint_1), $
           Longitude_Footprint_2 : temporary(Longitude_Footprint_2), $
           Longitude_Footprint_3 : temporary(Longitude_Footprint_3), $
           Longitude_Footprint_4 : temporary(Longitude_Footprint_4), $
           Scan : temporary(scan), $
           sequence : temporary(sequence), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time) }

    geo_attributes = { $
                 BoresightAzimuth : temporary(attrib0), $
                 BoresightNadirAngle : temporary(attrib1), $
                 BoresightNadirAngleUnc : temporary(attrib2), $
                 Latitude : temporary(attrib7), $
                 Latitude_Footprint_1 : temporary(attrib8), $
                 Latitude_Footprint_2 : temporary(attrib9), $
                 Latitude_Footprint_3 : temporary(attrib10), $
                 Latitude_Footprint_4 : temporary(attrib11), $
                 LocalSolarTime : temporary(attrib12), $
                 Longitude : temporary(attrib13), $
                 Longitude_Footprint_1 : temporary(attrib14), $
                 Longitude_Footprint_2 : temporary(attrib15), $
                 Longitude_Footprint_3 : temporary(attrib16), $
                 Longitude_Footprint_4 : temporary(attrib17), $
                 Scan : temporary(attrib18), $
                 sequence : temporary(attrib19), $
                 SolarZenithAngle : temporary(attrib20), $
                 SurfaceElevStandardDeviation :  temporary(attrib21), $
                 Tgt_SpacecraftAzimuth : temporary(attrib23), $
                 Tgt_SpacecraftZenith : temporary(attrib24), $
                 Time : temporary(attrib25) }
endif
if version eq 3 and type eq 'Nadir' then begin
    geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
           DayNightFlag : temporary(DayNightFlag), $
           DominantSurfaceType : temporary(DominantSurfaceType), $
           Latitude : temporary(Latitude), $
           Latitude_Footprint_1 : temporary(Latitude_Footprint_1), $
           Latitude_Footprint_2 : temporary(Latitude_Footprint_2), $
           Latitude_Footprint_3 : temporary(Latitude_Footprint_3), $
           Latitude_Footprint_4 : temporary(Latitude_Footprint_4), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Longitude_Footprint_1 : temporary(Longitude_Footprint_1), $
           Longitude_Footprint_2 : temporary(Longitude_Footprint_2), $
           Longitude_Footprint_3 : temporary(Longitude_Footprint_3), $
           Longitude_Footprint_4 : temporary(Longitude_Footprint_4), $
           Scan : temporary(scan), $
           sequence : temporary(sequence), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           SurfaceTypeFootprint : temporary(SurfaceTypeFootprint), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time), $
           UTCTime : temporary(UTCTime) $
           }

    geo_attributes = { $
                 BoresightAzimuth : temporary(attrib0), $
                 BoresightNadirAngle : temporary(attrib1), $
                 BoresightNadirAngleUnc : temporary(attrib2), $
                 DayNightFlag : temporary(attrib5), $
                 DominantSurfaceType : temporary(attrib6), $
                 Latitude : temporary(attrib7), $
                 Latitude_Footprint_1 : temporary(attrib8), $
                 Latitude_Footprint_2 : temporary(attrib9), $
                 Latitude_Footprint_3 : temporary(attrib10), $
                 Latitude_Footprint_4 : temporary(attrib11), $
                 LocalSolarTime : temporary(attrib12), $
                 Longitude : temporary(attrib13), $
                 Longitude_Footprint_1 : temporary(attrib14), $
                 Longitude_Footprint_2 : temporary(attrib15), $
                 Longitude_Footprint_3 : temporary(attrib16), $
                 Longitude_Footprint_4 : temporary(attrib17), $
                 Scan : temporary(attrib18), $
                 sequence : temporary(attrib19), $
                 SolarZenithAngle : temporary(attrib20), $
                 SurfaceElevStandardDeviation :  temporary(attrib21), $
                 SurfaceTypeFootprint :  temporary(attrib22), $
                 Tgt_SpacecraftAzimuth : temporary(attrib23), $
                 Tgt_SpacecraftZenith : temporary(attrib24), $
                 Time : temporary(attrib25), $
                 UTCTime : temporary(attrib26) $
                 }
endif
if version eq 3 and type eq 'Limb' then begin
    geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
           BoresightTangentHeight : temporary(BoresightTangentHeight), $
           BoresightTangentHeightUnc : temporary(BoresightTangentHeightUnc), $
           DayNightFlag : temporary(DayNightFlag), $
           Latitude : temporary(Latitude), $
           Latitude_Footprint_1 : temporary(Latitude_Footprint_1), $
           Latitude_Footprint_2 : temporary(Latitude_Footprint_2), $
           Latitude_Footprint_3 : temporary(Latitude_Footprint_3), $
           Latitude_Footprint_4 : temporary(Latitude_Footprint_4), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Longitude_Footprint_1 : temporary(Longitude_Footprint_1), $
           Longitude_Footprint_2 : temporary(Longitude_Footprint_2), $
           Longitude_Footprint_3 : temporary(Longitude_Footprint_3), $
           Longitude_Footprint_4 : temporary(Longitude_Footprint_4), $
           Scan : temporary(scan), $
           sequence : temporary(sequence), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           SurfaceTypeFootprint : temporary(SurfaceTypeFootprint), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time), $
           UTCTime : temporary(UTCTime) $
           }

    geo_attributes = { $
                 BoresightAzimuth : temporary(attrib0), $
                 BoresightNadirAngle : temporary(attrib1), $
                 BoresightNadirAngleUnc : temporary(attrib2), $
                 BoresightTangentHeight : temporary(attrib3), $
                 BoresightTangentHeightUnc : temporary(attrib4), $
                 DayNightFlag : temporary(attrib5), $
                 Latitude : temporary(attrib7), $
                 Latitude_Footprint_1 : temporary(attrib8), $
                 Latitude_Footprint_2 : temporary(attrib9), $
                 Latitude_Footprint_3 : temporary(attrib10), $
                 Latitude_Footprint_4 : temporary(attrib11), $
                 LocalSolarTime : temporary(attrib12), $
                 Longitude : temporary(attrib13), $
                 Longitude_Footprint_1 : temporary(attrib14), $
                 Longitude_Footprint_2 : temporary(attrib15), $
                 Longitude_Footprint_3 : temporary(attrib16), $
                 Longitude_Footprint_4 : temporary(attrib17), $
                 Scan : temporary(attrib18), $
                 sequence : temporary(attrib19), $
                 SolarZenithAngle : temporary(attrib20), $
                 SurfaceElevStandardDeviation :  temporary(attrib21), $
                 SurfaceTypeFootprint :  temporary(attrib22), $
                 Tgt_SpacecraftAzimuth : temporary(attrib23), $
                 Tgt_SpacecraftZenith : temporary(attrib24), $
                 Time : temporary(attrib25), $
                 UTCTime : temporary(attrib26) $
                 }
endif

if version ge 4 and type eq 'Nadir' then begin
    geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
           DayNightFlag : temporary(DayNightFlag), $
           DominantSurfaceType : temporary(DominantSurfaceType), $
           Latitude : temporary(Latitude), $
           Latitude_Footprint_1 : temporary(Latitude_Footprint_1), $
           Latitude_Footprint_2 : temporary(Latitude_Footprint_2), $
           Latitude_Footprint_3 : temporary(Latitude_Footprint_3), $
           Latitude_Footprint_4 : temporary(Latitude_Footprint_4), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Longitude_Footprint_1 : temporary(Longitude_Footprint_1), $
           Longitude_Footprint_2 : temporary(Longitude_Footprint_2), $
           Longitude_Footprint_3 : temporary(Longitude_Footprint_3), $
           Longitude_Footprint_4 : temporary(Longitude_Footprint_4), $
           Scan : temporary(scan), $
           sequence : temporary(sequence), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           SurfaceTypeFootprint : temporary(SurfaceTypeFootprint), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time) $
           }

    geo_attributes = { $
                 BoresightAzimuth : temporary(attrib0), $
                 BoresightNadirAngle : temporary(attrib1), $
                 BoresightNadirAngleUnc : temporary(attrib2), $
                 DayNightFlag : temporary(attrib5), $
                 DominantSurfaceType : temporary(attrib6), $
                 Latitude : temporary(attrib7), $
                 Latitude_Footprint_1 : temporary(attrib8), $
                 Latitude_Footprint_2 : temporary(attrib9), $
                 Latitude_Footprint_3 : temporary(attrib10), $
                 Latitude_Footprint_4 : temporary(attrib11), $
                 LocalSolarTime : temporary(attrib12), $
                 Longitude : temporary(attrib13), $
                 Longitude_Footprint_1 : temporary(attrib14), $
                 Longitude_Footprint_2 : temporary(attrib15), $
                 Longitude_Footprint_3 : temporary(attrib16), $
                 Longitude_Footprint_4 : temporary(attrib17), $
                 Scan : temporary(attrib18), $
                 sequence : temporary(attrib19), $
                 SolarZenithAngle : temporary(attrib20), $
                 SurfaceElevStandardDeviation :  temporary(attrib21), $
                 SurfaceTypeFootprint :  temporary(attrib22), $
                 Tgt_SpacecraftAzimuth : temporary(attrib23), $
                 Tgt_SpacecraftZenith : temporary(attrib24), $
                 Time : temporary(attrib25) $
                 }
endif
if version ge 4 and type eq 'Limb' then begin
    geo_data = { $           
           BoresightAzimuth : temporary(BoresightAzimuth), $
           BoresightNadirAngle : temporary(BoresightNadirAngle), $
           BoresightNadirAngleUnc : temporary(BoresightNadirAngleUnc), $
           BoresightTangentHeight : temporary(BoresightTangentHeight), $
           BoresightTangentHeightUnc : temporary(BoresightTangentHeightUnc), $
           DayNightFlag : temporary(DayNightFlag), $
           Latitude : temporary(Latitude), $
           Latitude_Footprint_1 : temporary(Latitude_Footprint_1), $
           Latitude_Footprint_2 : temporary(Latitude_Footprint_2), $
           Latitude_Footprint_3 : temporary(Latitude_Footprint_3), $
           Latitude_Footprint_4 : temporary(Latitude_Footprint_4), $
           LocalSolarTime : temporary(LocalSolarTime), $
           Longitude : temporary(Longitude), $
           Longitude_Footprint_1 : temporary(Longitude_Footprint_1), $
           Longitude_Footprint_2 : temporary(Longitude_Footprint_2), $
           Longitude_Footprint_3 : temporary(Longitude_Footprint_3), $
           Longitude_Footprint_4 : temporary(Longitude_Footprint_4), $
           Scan : temporary(scan), $
           sequence : temporary(sequence), $
           SolarZenithAngle : temporary(SolarZenithAngle), $
           SurfaceElevStandardDeviation : temporary(SurfaceElevStandardDeviation), $
           SurfaceTypeFootprint : temporary(SurfaceTypeFootprint), $
           Tgt_SpacecraftAzimuth : temporary(Tgt_SpacecraftAzimuth), $
           Tgt_SpacecraftZenith : temporary(Tgt_SpacecraftZenith), $
           Time : temporary(Time) $
           }

    geo_attributes = { $
                 BoresightAzimuth : temporary(attrib0), $
                 BoresightNadirAngle : temporary(attrib1), $
                 BoresightNadirAngleUnc : temporary(attrib2), $
                 BoresightTangentHeight : temporary(attrib3), $
                 BoresightTangentHeightUnc : temporary(attrib4), $
                 DayNightFlag : temporary(attrib5), $
                 Latitude : temporary(attrib7), $
                 Latitude_Footprint_1 : temporary(attrib8), $
                 Latitude_Footprint_2 : temporary(attrib9), $
                 Latitude_Footprint_3 : temporary(attrib10), $
                 Latitude_Footprint_4 : temporary(attrib11), $
                 LocalSolarTime : temporary(attrib12), $
                 Longitude : temporary(attrib13), $
                 Longitude_Footprint_1 : temporary(attrib14), $
                 Longitude_Footprint_2 : temporary(attrib15), $
                 Longitude_Footprint_3 : temporary(attrib16), $
                 Longitude_Footprint_4 : temporary(attrib17), $
                 Scan : temporary(attrib18), $
                 sequence : temporary(attrib19), $
                 SolarZenithAngle : temporary(attrib20), $
                 SurfaceElevStandardDeviation :  temporary(attrib21), $
                 SurfaceTypeFootprint :  temporary(attrib22), $
                 Tgt_SpacecraftAzimuth : temporary(attrib23), $
                 Tgt_SpacecraftZenith : temporary(attrib24), $
                 Time : temporary(attrib25) $
                 }
endif

;
; get 'Additional' information
;
aid = h5g_open(hdfid,'ADDITIONAL')
faaid = h5g_open(aid,'FILE_ATTRIBUTES')

if read_ph ne 0 then begin
    dataid = h5d_open(faaid,'ProductionHistory')
    ProductionHistory = h5d_read(dataid)
    h5d_close,dataid
endif else begin
    ProductionHistory = ''
endelse

id = h5a_open_name(faaid,'InstrumentName')
InstrumentName = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'ProcessLevel')
ProcessLevel = h5a_read(id)
h5a_close,id

if version eq 1 then begin
    id = h5a_open_name(faaid,'TAI93AtOzOfGranule')
    TAI93At0zOfGranule = h5a_read(id)
    h5a_close,id
endif
if version ge 2 then begin
    id = h5a_open_name(faaid,'TAI93At0zOfGranule')
    TAI93At0zOfGranule = h5a_read(id)
    h5a_close,id
endif   
 
id = h5a_open_name(faaid,'GlobalSurveyNumber')
GlobalSurveyNumber = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'GranuleMonth')
GranuleMonth = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'GranuleDay')
GranuleDay = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'GranuleYear')
GranuleYear = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'SurveyMode')
SurveyMode = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'PGEVersion')
PGEVersion = h5a_read(id)
h5a_close,id

if version ge 2 then begin
    id = h5a_open_name(faaid,'Command_Seq_ID')
    Command_Seq_ID = h5a_read(id)
    h5a_close,id
endif   


h5g_close,faaid
h5g_close,aid
h5g_close,hdfid

if version eq 1 then begin
    file_attributes = { $
                  ProductionHistory : temporary(ProductionHistory), $
                  InstrumentName : temporary(InstrumentName), $
                  ProcessLevel : temporary(ProcessLevel), $
                  TAI93At0zOfGranule : temporary(TAI93At0zOfGranule), $
                  GlobalSurveyNumber : temporary(GlobalSurveyNumber), $
                  GranuleMonth : temporary(GranuleMonth), $
                  GranuleDay : temporary(GranuleDay), $
                  GranuleYear : temporary(GranuleYear), $
                  SurveyMode : temporary(SurveyMode), $
                  PGEVersion : temporary(PGEVersion) }
endif

if version ge 2 then begin
    file_attributes = { $
                  ProductionHistory : temporary(ProductionHistory), $
                  InstrumentName : temporary(InstrumentName), $
                  ProcessLevel : temporary(ProcessLevel), $
                  TAI93At0zOfGranule : temporary(TAI93At0zOfGranule), $
                  GlobalSurveyNumber : temporary(GlobalSurveyNumber), $
                  GranuleMonth : temporary(GranuleMonth), $
                  GranuleDay : temporary(GranuleDay), $
                  GranuleYear : temporary(GranuleYear), $
                  SurveyMode : temporary(SurveyMode), $
                  PGEVersion : temporary(PGEVersion), $
                  Command_Seq_ID : temporary(Command_Seq_ID)    }
endif



hdfinfid = h5g_open(fid,'HDFEOS INFORMATION')

dataid = h5d_open(hdfinfid,'StructMetadata.0')
StructMetadata = h5d_read(dataid)
h5d_close,dataid

dataid = h5d_open(hdfinfid,'coremetadata')
coremetadata = h5d_read(dataid)
h5d_close,dataid

id = h5a_open_name(hdfinfid,'HDFEOSVersion')
HDFEOSVersion = h5a_read(id)
h5a_close,id

h5g_close,hdfinfid
h5f_close,fid

hdfeos_data = { $
              StructMetadata : temporary(StructMetadata), $
              coremetadata : temporary(coremetadata), $
              HDFEOSVersion : temporary(HDFEOSVersion) }


return

end
