;+
;
;                      Tropospheric Emission
;                          Spectrometer
;                             (TES)
;
;                         Copyright 2008-2011
;              California Institute of Technology.
;                       ALL RIGHTS RESERVED.
;           U.S. Government Sponsorship acknowledged.
;
;           Any commercial use must be negotiated with the
;           Office of Technology Transfer at the California
;           Institute of Technology.
;
; 
;  This software may be subject to U.S. export control laws. 
;  By accepting this software, the user agrees to comply with all 
;  applicable U.S. export laws and regulations. User has the 
;  responsibility to obtain export licenses, or other export 
;  authority as may be required before exporting such information 
;  to foreign countries or providing access to foreign persons.
;
;
; read_l2_hdf_field
;
; PURPOSE:
;	This routine reads and returns a single data field from a TES
;	L2 nadir HDF product file.  This allows for obtaining one or a
;	few (with multiple calls) of the data fields quickly when the
;	all of the fields are not needed.  To read the complete
;	contents of a L2 file, use read_l2_hdf
;
; CALL:
; 	result = read_l2_hdf_field(file,fieldname,$
;		   attributes=attributes, $
;		   swath_data=swath_data, $
;		   file_attributes=file_attributes, $
;		   status=status) 
;
; INPUTS:
;	file = name of the file to be read (string)
; 	fieldname = name of data field to be returned (string)
;	
;		The possible data fields to return 
;		are:
; 
;                species  (i.e. O3 [for ozone] in file)
;                Precision (in file this is 'species'Precision .i.e O3Precision)
;                VerticalResolution (in file this is
;                	'species'VerticalResolution,
;                	i.e O3VerticalResolution) (F01,F02)
;                	F03-this is just 'VerticalResolution' in the
;                	file
;		 AirDensity (F03)
;		 Altitude
;		 AverageCloudEffOpticalDepth (F03)
;                AveragingKernel
;                AveragingKernelDiagonal (F03)
;		 Calibration_QA (F03)
;		 CloudEffectiveOpticalDepth (F02+)
;		 CloudEffectiveOpticalDepthError (F02+)
;                CloudTopHeight (F01)
;                CloudTopPressure
;                CloudTopPressureError (F02+)
;		 CloudVariability_QA (F03)
;                ConstraintVector
;                DegreesOfFreedomForSignal
;                DeviationVsRetrievalCovariance
;		 H2O_HDO_Corr_QA (F04+)
;                InformationContent
;		 Initial (F03)
;		 KDotDL_QA (F03)
;		 LDotDL_QA (F03)
;                MaxNumIterations
;                MeasurementErrorCovariance
;                NumberIterPerformed (in versions F01-F02 in the file this 
;                		      NumbIterPerformed, but is still
;                		      returned as NumberIterPerformed)
;		 O3_Ccurve_QA (F05+) (Nadir Only)
;		 ObservationErrorCovariance (F03)
;                Pressure
;                RadianceResidualMax
;                RadianceResidualMean
;                RadianceResidualRMS
;                Scan_Averaged_Count (Nadir Only)
;                SpeciesRetrievalConverged
;                SpeciesRetrievalQuality (F02+)
;		 SurfaceEmissMean_QA (F03)  (Nadir Only)
;		 SurfaceEmissionLayer_QA (F04)  (Nadir Only)
;		 SurfaceTempConstraint (F03) (Nadir Only)
;		 SurfaceTempDegreesOfFreedom (F03) (Nadir Only)
;                SurfaceTempError (Nadir Only) 
;		 SurfaceTempInitial (F03) (Nadir Only)
;		 SurfaceTempObservationError (F03) (Nadir Only)
;		 SurfaceTempPrecision (F03) (Nadir Only)
;		 SurfaceTempVsApriori_QA (F03) (Nadir Only)
;		 SurfaceTempVsAtmTemp_QA (F03) (Nadir Only)
;                SurfaceTemperature (Nadir Only)
;                SystematicErrorCovariance (F01-F02)
;                TotalColumnDensity
;                TotalColumnDensityError
;                TotalColumnDensityInitial (F03)
;                TotalError
;                TotalErrorCovariance
;	    	 UTCTime (F04) 	 
;		 
;	
;		where (species) is the retrieved species or
;		temperature (O3, CH4, CO, H2O, ATM, ...)
;
;		the species (or temperature) may be specfied with
;		shorthand 'gas'.  The (species)Precision and 
;		(species)VerticalResolution may be specified without
;		the species prefix.
;		All fields may be specified with the minimum
;		non-ambiguous number of characters and are case
;		insensitive.
;
; KEYWORD OUTPUT:
;
;	attributes = the attributes associated with the data field in
;		a structure. An attributes
;		structure containts the following fields:
;
;		_FillValue
;		MissingValue
;		Title
;		Units
;	 	UniqueFieldDefinition
;
;	swath_data = swath level attribute data :
;               PressureGrid (Pressure in the file)
;	  	CloudFrequency  (F02+)
;		EmissivityWavenumber (F02+)
;               VerticalCoordinate 
;               Scan_Resolution
;
;	file_attributes = a structure containing the information
;		contained in the file attributes group in the
;		additional group.  The fields in this structure are:
;                  ProductionHistory 
;                  InstrumentName 
;                  ProcessLevel 
;                  TAI93AtOzOfGranule (F01- TAI93 At "capital o" Of
;                                          Granule) (a typo)
;			(obsolete--this field will be returned
;			labeled TAI93At0zOfGranule for all versions)
;                  TAI93At0zOfGranule (F02- TAI93 At "zero" Of Granule)
;                  GlobalSurveyNumber
;                  GranuleMonth
;                  GranuleDay
;                  GranuleYear
;                  SurveyMode
;                  PGEVersion
;		   Command_Seq_ID (F02+)
;
;	status = Contains success or failure status of the read
;		 0 for success
;		 1 if file does not exist
;		 2 the file name does not have F01, F02, or F03 in name
;		 3 the file is not HDF 5
;		 4 Nadir or Limb type not found in filename
;		 5 the file name does not have the expected .he5 extension
;		 6 Ambiguous field name specified
;		 7 Unknown field name specified
;		 8 File is ANCILLARY or SUMMARY type file
;		 9 'gas' not found(should never occur?)
;		10 No fieldname or index specified
;	        11 Fieldname not supported by file version
;	        12 IDL version incompatibility
;
;		For all non-zero status values, 0(zero) is returned as 
;		the result.
;
; ORIGINAL AUTHOR:
;	Brendan Fisher, JPL, 08/2005
;
; UPDATE HISTORY:
;	Brendan Fisher, JPL, 09/2005 Added file_attributes keyword to
;		make an easy way to get the observation date.
;	11/2005, Brendan Fisher, JPL
;	-Changed to default to not reading production history
;	-updated to read F02_00 files (these should normally not be
;		encountered anymore)
;	12/2005, Brendan Fisher, JPL
;	-updated to read F02_01 files
;	01/2006, Brendan Fisher, JPL
;	-updated to F03
;	-added limb capability
;	- removed index keyword because of new code methodology and
;         backwards compatibility issues
;	2006/08/22, Brendan Fisher, JPL, started update to F04
;	-added check to reject SUMMARY and ANCILLARY type files
;	2006/09/18, Brendan Fisher, JPL, added IDL version checking.
;
;
;-


function read_l2_hdf_field,file,fieldname, $
  status=status,attributes=attributes,file_attributes=file_attributes, $
  read_ph=read_ph,swath_data=swath_data


status=0
if n_elements(read_ph) eq 0 then read_ph = 0

; check for file existence
ff = file_search(file,count=count)
if count eq 0 then begin
    print,'Cannot find file:'
    print,file
    status = 1
    return,0
endif
; check type
apos = strpos(file,'ANCILLARY')
spos = strpos(file,'SUMMARY')
if apos ne -1 or spos ne -1 then begin
    print,'This reader does not work for ANCILLARY or SUMMARY product files.'
    print,'File name: ',file
    status=8
    return,0
end
; check version
pos1 = strpos(file,'_F01_')
pos2 = strpos(file,'_F02_')
pos3 = strpos(file,'_F03_')
pos4 = strpos(file,'_F04_')
pos5 = strpos(file,'_F05_')
if pos1 eq -1 and pos2 eq -1 and pos3 eq -1 and pos4 eq -1 and pos5 eq -1 then begin
    print,'Expected version _F01_, _F02_,_F03_,_F04_, or _F05_ in file name not found.'
    print,file
    print,'This file format not supported.'
    status = 2
    return,0
endif
if pos1 ne -1 then version = 1
if pos2 ne -1 then version = 2
if pos3 ne -1 then version = 3
if pos4 ne -1 then version = 4
if pos5 ne -1 then version = 5
; check to see if file is an HDF5 
if not h5f_is_hdf5(file) then begin
    print,'File is not HDF 5:'
    print,file
    status=3
    return,0
endif
;check for .he5 extension
;
len = strlen(file)
exten = strmid(file,len-3,3)
if exten ne 'he5' then begin
    print,'Expected .he5 file name extension not found.'
    print,file
    status=5
    return,0
endif
; check IDL version
iversion = !version
idl_version = float(iversion.release)
if version ge 4 and idl_version lt 6.3 then begin
    print,'IDL v6.3+ required for TES format version F04+ files.'
    status = 12
    return,0
endif
if idl_version lt 6.1 and version le 3 then begin
    print,'IDL v6.1+ required for TES format version F01, F02, and F03 files.'
    status = 12
    return,0
endif

; check for Nadir or Limb in name
posn = strpos(file,'Nadir')
posl = strpos(file,'Limb')
if posn eq -1 and posl eq -1 then begin
    print,'Expected Nadir or Limb in file name not found.'
    print,file
    status = 4
    return,0
endif
if posn ne -1 then type = 'Nadir'
if posl ne -1 then type = 'Limb'

; get species from file name
;
; file name always has '_L2-species-'
;
pos = strpos(file,'_L2-')
pos2 = strpos(file,'-',pos+4)
gas=strmid(file,pos+4,pos2-(pos+4))
if gas eq 'ATM' then gas = 'TATM'


fid = h5f_open(file)
;hdfid = h5g_open(fid,'HDFEOS')
;swid = h5g_open(hdfid,'SWATHS')


; get members from swath

nfields = h5g_get_nmembers(fid,'/HDFEOS/SWATHS/'+gas+type+'Swath/Data Fields/')
field_names = strarr(nfields)
for i=0,nfields-1 do field_names[i] = $
  h5g_get_member_name(fid,'/HDFEOS/SWATHS/'+gas+type+'Swath/Data Fields/',i)

               
; does fieldname exist
if n_elements(fieldname) ne 0 then begin
; prefix gas name to 'Precision' or 'VerticalResolution' if they are input
    if strlowcase(fieldname) eq 'precision' then $
      fieldname = gas + fieldname
    if strlowcase(fieldname) eq 'verticalresolution' and version le 2 then $
      fieldname = gas + fieldname
; if 'gas' input, change to actual species name
    gasonly = 0
    if strlowcase(fieldname) eq 'gas' then begin
        fieldname = gas
        gasonly = 1
    endif
; try to find fieldname in list
    inlen = strlen(fieldname)
    if gasonly then begin
        match = where(field_names eq fieldname,count)
        if count eq -1 then begin
            print,'Gas not found??!'
            status=9
            return,0
        endif
    endif else begin
        res = strcmp(fieldname,field_names,inlen,/fold_case)
        match = where(res eq 1,count)
    endelse
    if count gt 1 then begin
        good = where(strlowcase(fieldname) eq strlowcase(field_names[match]),cc)
        if cc gt 1 then begin
            print,'Ambigious Field name: ',fieldname
            status=6
            return,0
        endif else begin
            if cc eq 1 then $
              match = match[good[0]] $
            else begin
                print,'Ambigious Field name: ',fieldname
                status=6
                return,0
            endelse
        endelse
    endif
    if count eq 0 then begin
        print,'Unknown Field name: ',fieldname
        status=7
        return,0
    endif
    foundname = field_names[match[0]]
endif

if n_elements(foundname) eq 0 then begin
    print,'No field specified!'
    status=10
    return,0
endif
 

;
; get data
;
; open file to data fields
;
; open file
fid = h5f_open(file)
; open hdfeos group
hdfid = h5g_open(fid,'HDFEOS')
; open swaths group
swid = h5g_open(hdfid,'SWATHS')
; open swath group
specid = h5g_open(swid,gas+type+'Swath')
; open data fields group
fieldid = h5g_open(specid,'Data Fields')
; open data field
dataid = h5d_open(fieldid,foundname)
; get data
data = h5d_read(dataid)
; get attributes
if foundname eq 'UTCTime' and version ge 4 then $
  get_data_attributes,dataid,size(data,/type),attributes,/utctime $
else $ 
  get_data_attributes,dataid,size(data,/type),attributes
  
; close ids to swath
h5d_close,dataid
h5g_close,fieldid
;
; get swath level attributes
;
id = h5a_open_name(specid,'Pressure')
pressuregrid = h5a_read(id)
h5a_close,id

id = h5a_open_name(specid,'VerticalCoordinate')
VerticalCoordinate = h5a_read(id)
h5a_close,id

if version ge 2 then begin
    id = h5a_open_name(specid,'CloudFrequency')
    CloudFrequency = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'EmissivityWavenumber')
    EmissivityWavenumber = h5a_read(id)
    h5a_close,id
endif

id = h5a_open_name(specid,'Scan_Resolution')
Scan_Resolution = h5a_read(id)
h5a_close,id

if version eq 1 then begin
    swath_data = { $
                 pressuregrid : pressuregrid, $
                 verticalcoordinate : verticalcoordinate, $
                 Scan_Resolution : Scan_Resolution $
                 }
endif else begin
    swath_data = { $
                 pressuregrid : temporary(pressuregrid), $
                 CloudFrequency : temporary(CloudFrequency), $
                 EmissivityWavenumber : temporary(EmissivityWavenumber), $
                 verticalcoordinate : temporary(verticalcoordinate), $
                 Scan_Resolution : temporary(Scan_Resolution) $
                 }
endelse
; close more ids
h5g_close,specid
h5g_close,swid

;
; get 'Additional' information
;
aid = h5g_open(hdfid,'ADDITIONAL')
faaid = h5g_open(aid,'FILE_ATTRIBUTES')

if read_ph ne 0 then begin
    dataid = h5d_open(faaid,'ProductionHistory')
    ProductionHistory = h5d_read(dataid)
    h5d_close,dataid
endif else begin
    ProductionHistory = ''
endelse

id = h5a_open_name(faaid,'InstrumentName')
InstrumentName = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'ProcessLevel')
ProcessLevel = h5a_read(id)
h5a_close,id

if version eq 1 then begin
    id = h5a_open_name(faaid,'TAI93AtOzOfGranule')
    TAI93At0zOfGranule = h5a_read(id)
    h5a_close,id
endif
if version ge 2 then begin
    id = h5a_open_name(faaid,'TAI93At0zOfGranule')
    TAI93At0zOfGranule = h5a_read(id)
    h5a_close,id
endif   

id = h5a_open_name(faaid,'GlobalSurveyNumber')
GlobalSurveyNumber = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'GranuleMonth')
GranuleMonth = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'GranuleDay')
GranuleDay = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'GranuleYear')
GranuleYear = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'SurveyMode')
SurveyMode = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'PGEVersion')
PGEVersion = h5a_read(id)
h5a_close,id

if version ge 2 then begin
    id = h5a_open_name(faaid,'Command_Seq_ID')
    Command_Seq_ID = h5a_read(id)
    h5a_close,id
endif   

; close more ids
h5g_close,faaid
h5g_close,aid
h5g_close,hdfid


if version eq 1 then begin
    file_attributes = { $
                  ProductionHistory : temporary(ProductionHistory), $
                  InstrumentName : temporary(InstrumentName), $
                  ProcessLevel : temporary(ProcessLevel), $
                  TAI93At0zOfGranule : temporary(TAI93At0zOfGranule), $
                  GlobalSurveyNumber : temporary(GlobalSurveyNumber), $
                  GranuleMonth : temporary(GranuleMonth), $
                  GranuleDay : temporary(GranuleDay), $
                  GranuleYear : temporary(GranuleYear), $
                  SurveyMode : temporary(SurveyMode), $
                  PGEVersion : temporary(PGEVersion) }
endif

if version ge 2 then begin
    file_attributes = { $
                  ProductionHistory : temporary(ProductionHistory), $
                  InstrumentName : temporary(InstrumentName), $
                  ProcessLevel : temporary(ProcessLevel), $
                  TAI93At0zOfGranule : temporary(TAI93At0zOfGranule), $
                  GlobalSurveyNumber : temporary(GlobalSurveyNumber), $
                  GranuleMonth : temporary(GranuleMonth), $
                  GranuleDay : temporary(GranuleDay), $
                  GranuleYear : temporary(GranuleYear), $
                  SurveyMode : temporary(SurveyMode), $
                  PGEVersion : temporary(PGEVersion), $
                  Command_Seq_ID : temporary(Command_Seq_ID)    }
endif

; close file
h5f_close,fid

return,data

end
