;+
;
;                      Tropospheric Emission
;                          Spectrometer
;                             (TES)
;
;                         Copyright 2008-2011
;              California Institute of Technology.
;                       ALL RIGHTS RESERVED.
;           U.S. Government Sponsorship acknowledged.
;
;           Any commercial use must be negotiated with the
;           Office of Technology Transfer at the California
;           Institute of Technology.
;
; 
;  This software may be subject to U.S. export control laws. 
;  By accepting this software, the user agrees to comply with all 
;  applicable U.S. export laws and regulations. User has the 
;  responsibility to obtain export licenses, or other export 
;  authority as may be required before exporting such information 
;  to foreign countries or providing access to foreign persons.
;
;
; read_l2_hdf_geofield
;
; PURPOSE:
;	This routine reads and returns a single geolocation field from a TES
;	L2 HDF product file.  This allows for obtaining one or a
;	few (with multiple calls) of the data fields quickly when the
;	all of the fields are not needed.  To read the complete
;	contents of a file, use read_l2_hdf
;
; CALL:
; 	result = read_l2_hdf_geofield(file,fieldname,$
;		   attributes=attributes,status=status,
;		   concatinate=concatinate) 
;
; INPUTS:
;	file = name of the file to be read (string)
; 	fieldname = name of data field to be returned (string)
;		The possible data fields to return 
;		are:
; 
;           BoresightAzimuth
;	    BoresightNadirAngle 
;           BoresightNadirAngleUnc
;	    BoresightTangentHeight (F03) (Limb Only)
;	    BoresightTangentHeightUnc (F03) (Limb Only)
;	    DayNightFlag (F03)
;	    DominantSurfaceType (F03)
;           Latitude
;           Latitude_Footprint_1
;           Latitude_Footprint_2
;           Latitude_Footprint_3
;           Latitude_Footprint_4
;           LocalSolarTime
;           Longitude
;           Longitude_Footprint_1
;           Longitude_Footprint_2
;           Longitude_Footprint_3
;           Longitude_Footprint_4
;	    Scan (F02+)
;	    Sequence (F02+)
;           SolarZenithAngle
;           SurfaceElevStandardDeviation
;	    SurfaceTypeFootprint (F03) 
;           Tgt_SpacecraftAzimuth
;           Tgt_SpacecraftZenith
;           Time
;	    UTCTime (F03) Note: in the HDF file this is stored as an
;	                 27xN array (where N is the number of observations
;	                 in this file) and 27 is the number of
;	                 characters in the string.  This program 
;			 will concatinate all of the characters into a
;			 single string and return a string array of N
;			 length if the concatinate keyword is set.
;			This entry has no _FillValue field in its attributes
;;
;		All fields may be specified with the minimum
;		non-ambiguous number of characters and are case
;		insensitive.
;
; KEYWORD INPUT:
;	concatinate : if set, concatinate the UTCTime array into a 
;			one dimensional array with the full UTCTime
;			string in each entry.
;
;
; KEYWORD OUTPUT:
;
;	attributes = the attributes associated with the data field in
;		a structure. An attributes
;		structure containts the following fields:
;
;		_FillValue
;		MissingValue
;		Title
;		Units
;	 	UniqueFieldDefinition
;
;	status = Contains success or failure status of the read
;		 0 for success
;		 1 if file does not exist
;		 2 the file name does not have F01_01 in name
;		 3 the file is not HDF 5
;		 4 Nadir or Limb type not found in filename
;		 5 the file name does not have the expected .he5 extension
;		 6 Ambiguous field name specified
;		 7 Unknown field name specified
;		 8 File is ANCILLARY or SUMMARY type file
;		10 No fieldname or index specified
;	        11 Fieldname not supported by file version
;	        12 IDL version incompatibility
;
;		For all non-zero status values, 0(zero) is returned as 
;		the result.
;
; ORIGINAL AUTHOR:
;	Brendan Fisher, JPL, 08/2005
;
; UPDATE HISTORY:
;	11/2005, Brendan Fisher, JPL
;	-updated to read F02_00 files
;	12/2005, Brendan Fisher, JPL
;	-updated to read F02_01 files
;	01/2006, Brendan Fisher, JPL
;	-updated to F03
;	-added limb capability
;	- removed index keyword because of new code methodology and
;         backwards compatibility issues
;	2006/04/03, Brendan Fisher, JPL, fixed bug in UTCTime
;	conversion & added keyword to enable conversion
;	2006/08/22, Brendan Fisher, JPL, started update to F04
;	-added check to reject SUMMARY and ANCILLARY type files
;	2006/09/18, Brendan Fisher, JPL, added IDL version checking.
;
;-


function read_l2_hdf_geofield,file,fieldname, $
  status=status,attributes=attributes,concatinate=concatinate

if n_elements(concatinate) eq 0 then concatinate = 0

status=0

; check for file existence
ff = file_search(file,count=count)
if count eq 0 then begin
    print,'Cannot find file:'
    print,file
    status = 1
    return,0
endif
; check type
apos = strpos(file,'ANCILLARY')
spos = strpos(file,'SUMMARY')
if apos ne -1 or spos ne -1 then begin
    print,'This reader does not work for ANCILLARY or SUMMARY product files.'
    print,'File name: ',file
    status=8
    return,0
end
; check version
pos1 = strpos(file,'_F01_')
pos2 = strpos(file,'_F02_')
pos3 = strpos(file,'_F03_')
pos4 = strpos(file,'_F04_')
pos5 = strpos(file,'_F05_')
if pos1 eq -1 and pos2 eq -1 and pos3 eq -1 and pos4 eq -1 and pos5 eq -1 then begin
    print,'Expected version _F01_, _F02_,_F04_, or _F05_ in file name not found.'
    print,file
    print,'This file format not supported.'
    status = 2
    return,0
endif
if pos1 ne -1 then version = 1
if pos2 ne -1 then version = 2
if pos3 ne -1 then version = 3
if pos4 ne -1 then version = 4
if pos5 ne -1 then version = 5

; check to see if file is an HDF5 
if not h5f_is_hdf5(file) then begin
    print,'File is not HDF 5:'
    print,file
    status=3
    return,0
endif
;check for .he5 extension
;
len = strlen(file)
exten = strmid(file,len-3,3)
if exten ne 'he5' then begin
    print,'Expected .he5 file name extension not found.'
    print,file
    status=5
    return,0
endif
; check IDL version
iversion = !version
idl_version = float(iversion.release)
if version ge 4 and idl_version lt 6.3 then begin
    print,'IDL v6.3+ required for TES format version F04+ files.'
    status = 12
    return,0
endif
if idl_version lt 6.1 and version le 3 then begin
    print,'IDL v6.1+ required for TES format version F01, F02, and F03 files.'
    status = 12
    return,0
endif
; check for Nadir or Limb in name
posn = strpos(file,'Nadir')
posl = strpos(file,'Limb')
if posn eq -1 and posl eq -1 then begin
    print,'Expected Nadir or Limb in file name not found.'
    print,file
    status = 4
    return,0
endif
if posn ne -1 then type = 'Nadir'
if posl ne -1 then type = 'Limb'

; get species from file name
;
; file name always has '_L2-species-'
;
pos = strpos(file,'_L2-')
pos2 = strpos(file,'-',pos+4)
gas=strmid(file,pos+4,pos2-(pos+4))
if gas eq 'ATM' then gas = 'TATM'

fid = h5f_open(file)
;hdfid = h5g_open(fid,'HDFEOS')
;swid = h5g_open(hdfid,'SWATHS')


; get members from swath

nfields = h5g_get_nmembers(fid,'/HDFEOS/SWATHS/'+gas+type+'Swath/Geolocation Fields/')
field_names = strarr(nfields)
for i=0,nfields-1 do field_names[i] = $
  h5g_get_member_name(fid,'/HDFEOS/SWATHS/'+gas+type+'Swath/Geolocation Fields/',i)



; does fieldname exist
if n_elements(fieldname) ne 0 then begin
    inlen = strlen(fieldname)
    res = strcmp(fieldname,field_names,inlen,/fold_case)
    match = where(res eq 1,count)
    if count gt 1 then begin
        good = where(strlowcase(fieldname) eq strlowcase(field_names[match]),cc)
        if cc gt 1 then begin
            print,'Ambigious Field name: ',fieldname
            status=6
            return,0
        endif else begin
            if cc eq 1 then $
              match = match[good[0]] $
            else begin
                print,'Ambigious Field name: ',fieldname
                status=6
                return,0
            endelse
        endelse
    endif
    if count eq 0 then begin
        print,'Unknown Field name: ',fieldname
        status=7
        return,0
    endif
    foundname = field_names[match[0]]
endif

if n_elements(foundname) eq 0 then begin
    print,'No field specified!'
    status=10
    return,0
endif
;
;print,'Found ',foundname
;return,0
; get data
;
; open file to data fields
;
fid = h5f_open(file)
hdfid = h5g_open(fid,'HDFEOS')
swid = h5g_open(hdfid,'SWATHS')
specid = h5g_open(swid,gas+type+'Swath')
fieldid = h5g_open(specid,'Geolocation Fields')
dataid = h5d_open(fieldid,foundname)
data = h5d_read(dataid)
; need to do special for UTCTime
if foundname ne 'UTCTime' then $
  get_data_attributes,dataid,size(data,/type),attributes $
else $
  get_data_attributes,dataid,size(data,/type),attributes,/special

if concatinate and foundname eq 'UTCTime' then begin
    ua = temporary(data)
    ss = size(ua)
    UTCTime = strarr(ss[2])
    for i=0l,ss[2]-1 do begin
        utctime[i] = ua[0,i] +ua[1,i] +ua[2,i] +ua[3,i] +ua[4,i] +ua[5,i] + $
                     ua[6,i] +ua[7,i] +ua[8,i] +ua[9,i] +ua[10,i]+ua[11,i]+ $
                     ua[12,i]+ua[13,i]+ua[14,i]+ua[15,i]+ua[16,i]+ua[17,i]+ $
                     ua[18,i]+ua[19,i]+ua[20,i]+ua[21,i]+ua[22,i]+ua[23,i]+ $
                     ua[24,i]+ua[25,i]+ua[26,i]
    endfor
    data = temporary(utctime)
endif

h5d_close,dataid
h5g_close,fieldid
h5g_close,specid
h5g_close,swid
h5g_close,hdfid
h5f_close,fid

return,data

end
