;+
;
;                      Tropospheric Emission
;                          Spectrometer
;                             (TES)
;
;                         Copyright 2008-2011
;              California Institute of Technology.
;                       ALL RIGHTS RESERVED.
;           U.S. Government Sponsorship acknowledged.
;
;           Any commercial use must be negotiated with the
;           Office of Technology Transfer at the California
;           Institute of Technology.
;
; 
;  This software may be subject to U.S. export control laws. 
;  By accepting this software, the user agrees to comply with all 
;  applicable U.S. export laws and regulations. User has the 
;  responsibility to obtain export licenses, or other export 
;  authority as may be required before exporting such information 
;  to foreign countries or providing access to foreign persons.
;
;
; read_l2_hdf_supplemental
;
; PURPOSE:
;	Read a TES L2 HDF supplemntal data product file and
;	returns the data fields and geolocation for data
;	versions F01.  Data are returned in simple
;	structures of simple arrays with no processing, filtering,
;	etc.
;	Does NOT read species, ANCILLARY, or SUMMARY files
;	IDL 6.3+ REQUIRED to read version F01 files.
;
; CALL: 
; 	read_l2_hdf_supplimental,file, $
; 	  cfc11limb = cfc11limb, $
;	  cfc12limb = cfc12limb, $
;	  ch4limb = ch4limb, $
;	  co2limb = co2limb, $
;	  co2nadir = co2nadir, $
;	  n2olimb = n2olimb, $
;	  n2onadir = n2onadir, $
;	  o3limb = o3limb, $
;	  file_attributes = file_attributes,
;	  hdfeos_data = hdfeos_data, $
;	  read_ph = read_ph,status = status
;
; INPUTS:
;	file = name of file to be read (string)
;
; KEYWORD INPUT:
;	read_ph = if set, read production history.  If not set, the
;		production history will be a null string
;
;
; OUTPUTS:
;	
;	The possible swaths are:
;			CFC11Limb
;			CFC12Limb
;			CH4Limb
;			CO2Limb
;			CO2Nadir
;			N2OLimb
;			N2ONadir
;			O3Limb
;	Note: Not all swaths are present in all files
;
;		Each species swath structure contains the following data:
;			species (the species data)
;			pressure
;			latitude
;			longitude
;			scan
;			sequence
;			verticalcoordinate (swath level attribute)
;			cloudfrequency (swath level attribute)
;			emissivitywavenumber (swath level attribute)
;			scan_resolution (swath level attribute)
;			pressuregrid (swath level attribute)
;

;
; KEYWORD OUTPUT:
; 
;	file_attributes = a structure containing the information
;		contained in the file attributes group in the
;		additional group.  The fields in this structure are:
;                  ProductionHistory  (not read by default--see input
;                      keywords--this is added by the TES SIPS and is thus 
;                      not present in files produced by the TES SCF)
;                  InstrumentName 
;                  ProcessLevel 
;                  TAI93At0zOfGranule ( TAI93 At "zero" z Of Granule)
;                  GlobalSurveyNumber
;                  GranuleMonth
;                  GranuleDay
;                  GranuleYear
;                  SurveyMode
;                  PGEVersion
;		   Command_Seq_ID 
;
;	hdfeos_data = a structure that contains the information found 
;		in the HDFEOS Information group.  The fields are:
;              		StructMetadata 
;			coremetadata
;			HDFEOSVersion
;
;	status = Contains success or failure status of the read
;		 0 for success
;		 1 if file does not exist
;		 2 the file name does not have  F01 in name
;		 3 the file is not HDF 5
;		 4 'supplimental not found in the file name as expected
;		 5 the file name does not have the expected .he5
;		 extension
;		 6 File is ANCILLARY or SUMMARY species type file
;		 7 IDL version incompatability
;
; NOTE:  On any error except those listed, the program will stop in
; the standard IDL fashion at the error location.  No other error
; handling is built into this procedure other than those shown in the 
; status list.
;		 
; ORIGINAL AUTHOR:
;	Brendan Fisher, JPL, 12/2007
;
; UPDATE HISTORY:
;	2007/12, Brendan Fisher, JPL, initial version
;
;-



pro read_l2_hdf_supplemental,file,status=status, $
  cfc11limb = cfc11limb, $
  cfc12limb = cfc12limb, $
  ch4limb = ch4limb, $
  co2limb = co2limb, $
  co2nadir = co2nadir, $
  n2olimb = n2olimb, $
  n2onadir = n2onadir, $
  o3limb = o3limb, $ 
  list_swath=list_swath, $
  file_attributes=file_attributes, $
  hdfeos_data = hdfeos_data, $
  read_ph=read_ph


status=0
if n_elements(read_ph) eq 0 then read_ph = 0

; check for file existence
ff = file_search(file,count=count)
if count eq 0 then begin
    print,'Cannot find file:'
    print,file
    status = 1
    return
endif
; check type
apos = strpos(file,'ANCILLARY')
spos = strpos(file,'SUMMARY')
npos = strpos(file,'Nadir')
lpos = strpos(file,'Limb')
if apos ne -1 or spos ne -1 or npos ne -1 or lpos ne -1 then begin
    print,'This reader does not work for species, ANCILLARY or SUMMARY product files.'
    print,'File name: ',file
    status=6
    return
end
; check version
pos1 = strpos(file,'_F01_')
pos2 = strpos(file,'_F02_')
;pos3 = strpos(file,'_F03_')
;pos4 = strpos(file,'_F04_')
if pos1 eq -1 and pos2 eq -1 then begin
;if pos1 eq -1 and pos2 eq -1 and pos3 eq -1 and pos4 eq -1 then begin
    print,'Expected version _F01_,  in file name not found.'
    print,file
    print,'This file format not supported.'
    status = 2
    return
endif
if pos1 ne -1 then version = 1
if pos2 ne -1 then version = 2
;if pos3 ne -1 then version = 3
;if pos4 ne -1 then version = 4
; check to see if file is an HDF5 
if not h5f_is_hdf5(file) then begin
    print,'File is not HDF 5:'
    print,file
    status=3
    return
endif
; check for Nadir or Limb in name
poss = strpos(file,'SUPPLEMENTAL')

if poss eq -1  then begin
    print,'Expected SUPPLEMENTAL in file name not found.'
    print,file
    status = 4
    return
endif
;if type eq 'Limb' then version = 3
;check for .he5 extension
;
len = strlen(file)
exten = strmid(file,len-3,3)
if exten ne 'he5' then begin
    print,'Expected .he5 file name extension not found.'
    print,file
    status=5
    return
endif
; IDL version checking
iversion = !version
idl_version = float(iversion.release)
if version ge 1 and idl_version lt 6.3 then begin
    print,'IDL v6.3+ required for TES format version F01 files.'
    status = 7
    return
endif
;

;
; open file to data fields
;
fid = h5f_open(file)
hdfid = h5g_open(fid,'HDFEOS')
swid = h5g_open(hdfid,'SWATHS')
; check which swaths are in file
ngroup = h5g_get_nmembers(fid,'/HDFEOS/SWATHS/')
names = strarr(ngroup)
for i=0,ngroup-1 do names[i] = h5g_get_member_name(fid,'/HDFEOS/SWATHS/',i)
actual_swaths = names
if keyword_set(list_swaths) then print,names
rswaths = actual_swaths ; read all available (since file is small)
; swaths

;=====================================================================
swath = 'CFC11LimbSwath'
;=====================================================================
swpos = where(rswaths eq swath)
if swpos ne -1 then begin
; data fields
    specid = h5g_open(swid,swath)
    fieldid = h5g_open(specid,'Data Fields')
;
; actual data, alphabetically except for the gas
;
; get name from swath name
    lpos = strpos(swath,'Limb')
    npos = strpos(swath,'Nadir')
    if lpos ne -1 then gas = strmid(swath,0,lpos)
    if npos ne -1 then gas = strmid(swath,0,npos)
    
    dataid = h5d_open(fieldid,gas)
    species = h5d_read(dataid)
    ;get_data_attributes,dataid,size(species,/type),attrib0
    h5d_close,dataid


    dataid = h5d_open(fieldid,'Pressure')
    Pressure = h5d_read(dataid)
    get_data_attributes,dataid,size(Pressure,/type),attrib26
    h5d_close,dataid

    h5g_close,fieldid
    geoid = h5g_open(specid,'Geolocation Fields')

    dataid = h5d_open(geoid,'Latitude')
    Latitude = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Latitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'Longitude')
    Longitude = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Longitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'Scan')
    Scan = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Scan,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'Sequence')
    Sequence = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Sequence,/type),attrib7
    h5d_close,dataid

    h5g_close,geoid
;
; get swath level attributes
;
    id = h5a_open_name(specid,'Pressure')
    pressuregrid = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'VerticalCoordinate')
    VerticalCoordinate = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'CloudFrequency')
    CloudFrequency = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'EmissivityWavenumber')
    EmissivityWavenumber = h5a_read(id)
    h5a_close,id


    id = h5a_open_name(specid,'Scan_Resolution')
    Scan_Resolution = h5a_read(id)
    h5a_close,id

    cfc11limb = { $
                species :  temporary(species), $
                Pressure : temporary(Pressure), $
                latitude : temporary(latitude), $
                longitude : temporary(longitude), $
                scan : temporary(scan), $
                sequence : temporary(sequence), $
                CloudFrequency : temporary(CloudFrequency),$
                EmissivityWavenumber : temporary(EmissivityWavenumber), $
                pressuregrid : temporary(pressuregrid), $
                Scan_Resolution : temporary(Scan_Resolution), $
                VerticalCoordinate : temporary(VerticalCoordinate) $
                }
    h5g_close,specid
endif


;=====================================================================
swath = 'CFC12LimbSwath'
;=====================================================================
swpos = where(rswaths eq swath)
if swpos ne -1 then begin
; data fields
    specid = h5g_open(swid,swath)
    fieldid = h5g_open(specid,'Data Fields')
;
; actual data, alphabetically except for the gas
;
; get name from swath name
    lpos = strpos(swath,'Limb')
    npos = strpos(swath,'Nadir')
    if lpos ne -1 then gas = strmid(swath,0,lpos)
    if npos ne -1 then gas = strmid(swath,0,npos)
    
    dataid = h5d_open(fieldid,gas)
    species = h5d_read(dataid)
    ;get_data_attributes,dataid,size(species,/type),attrib0
    h5d_close,dataid


    dataid = h5d_open(fieldid,'Pressure')
    Pressure = h5d_read(dataid)
    get_data_attributes,dataid,size(Pressure,/type),attrib26
    h5d_close,dataid

    h5g_close,fieldid
    geoid = h5g_open(specid,'Geolocation Fields')

    dataid = h5d_open(geoid,'Latitude')
    Latitude = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Latitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'Longitude')
    Longitude = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Longitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'Scan')
    Scan = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Scan,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'Sequence')
    Sequence = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Sequence,/type),attrib7
    h5d_close,dataid

    h5g_close,geoid
;
; get swath level attributes
;
    id = h5a_open_name(specid,'Pressure')
    pressuregrid = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'VerticalCoordinate')
    VerticalCoordinate = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'CloudFrequency')
    CloudFrequency = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'EmissivityWavenumber')
    EmissivityWavenumber = h5a_read(id)
    h5a_close,id


    id = h5a_open_name(specid,'Scan_Resolution')
    Scan_Resolution = h5a_read(id)
    h5a_close,id

    cfc12limb = { $
                species :  temporary(species), $
                Pressure : temporary(Pressure), $
                latitude : temporary(latitude), $
                longitude : temporary(longitude), $
                scan : temporary(scan), $
                sequence : temporary(sequence), $
                CloudFrequency : temporary(CloudFrequency),$
                EmissivityWavenumber : temporary(EmissivityWavenumber), $
                pressuregrid : temporary(pressuregrid), $
                Scan_Resolution : temporary(Scan_Resolution), $
                VerticalCoordinate : temporary(VerticalCoordinate) $
                }
    h5g_close,specid
endif



;=====================================================================
swath = 'CH4LimbSwath'
;=====================================================================
swpos = where(rswaths eq swath)
if swpos ne -1 then begin
; data fields
    specid = h5g_open(swid,swath)
    fieldid = h5g_open(specid,'Data Fields')
;
; actual data, alphabetically except for the gas
;
; get name from swath name
    lpos = strpos(swath,'Limb')
    npos = strpos(swath,'Nadir')
    if lpos ne -1 then gas = strmid(swath,0,lpos)
    if npos ne -1 then gas = strmid(swath,0,npos)
    
    dataid = h5d_open(fieldid,gas)
    species = h5d_read(dataid)
    ;get_data_attributes,dataid,size(species,/type),attrib0
    h5d_close,dataid


    dataid = h5d_open(fieldid,'Pressure')
    Pressure = h5d_read(dataid)
    get_data_attributes,dataid,size(Pressure,/type),attrib26
    h5d_close,dataid

    h5g_close,fieldid
    geoid = h5g_open(specid,'Geolocation Fields')

    dataid = h5d_open(geoid,'Latitude')
    Latitude = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Latitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'Longitude')
    Longitude = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Longitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'Scan')
    Scan = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Scan,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'Sequence')
    Sequence = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Sequence,/type),attrib7
    h5d_close,dataid

    h5g_close,geoid
;
; get swath level attributes
;
    id = h5a_open_name(specid,'Pressure')
    pressuregrid = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'VerticalCoordinate')
    VerticalCoordinate = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'CloudFrequency')
    CloudFrequency = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'EmissivityWavenumber')
    EmissivityWavenumber = h5a_read(id)
    h5a_close,id


    id = h5a_open_name(specid,'Scan_Resolution')
    Scan_Resolution = h5a_read(id)
    h5a_close,id

    ch4limb = { $
                species :  temporary(species), $
                Pressure : temporary(Pressure), $
                latitude : temporary(latitude), $
                longitude : temporary(longitude), $
                scan : temporary(scan), $
                sequence : temporary(sequence), $
                CloudFrequency : temporary(CloudFrequency),$
                EmissivityWavenumber : temporary(EmissivityWavenumber), $
                pressuregrid : temporary(pressuregrid), $
                Scan_Resolution : temporary(Scan_Resolution), $
                VerticalCoordinate : temporary(VerticalCoordinate) $
                }
    h5g_close,specid
endif



;=====================================================================
swath = 'CO2LimbSwath'
;=====================================================================
swpos = where(rswaths eq swath)
if swpos ne -1 then begin
; data fields
    specid = h5g_open(swid,swath)
    fieldid = h5g_open(specid,'Data Fields')
;
; actual data, alphabetically except for the gas
;
; get name from swath name
    lpos = strpos(swath,'Limb')
    npos = strpos(swath,'Nadir')
    if lpos ne -1 then gas = strmid(swath,0,lpos)
    if npos ne -1 then gas = strmid(swath,0,npos)
    
    dataid = h5d_open(fieldid,gas)
    species = h5d_read(dataid)
    ;get_data_attributes,dataid,size(species,/type),attrib0
    h5d_close,dataid


    dataid = h5d_open(fieldid,'Pressure')
    Pressure = h5d_read(dataid)
    get_data_attributes,dataid,size(Pressure,/type),attrib26
    h5d_close,dataid

    h5g_close,fieldid
    geoid = h5g_open(specid,'Geolocation Fields')

    dataid = h5d_open(geoid,'Latitude')
    Latitude = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Latitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'Longitude')
    Longitude = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Longitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'Scan')
    Scan = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Scan,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'Sequence')
    Sequence = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Sequence,/type),attrib7
    h5d_close,dataid

    h5g_close,geoid
;
; get swath level attributes
;
    id = h5a_open_name(specid,'Pressure')
    pressuregrid = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'VerticalCoordinate')
    VerticalCoordinate = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'CloudFrequency')
    CloudFrequency = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'EmissivityWavenumber')
    EmissivityWavenumber = h5a_read(id)
    h5a_close,id


    id = h5a_open_name(specid,'Scan_Resolution')
    Scan_Resolution = h5a_read(id)
    h5a_close,id

    co2limb = { $
                species :  temporary(species), $
                Pressure : temporary(Pressure), $
                latitude : temporary(latitude), $
                longitude : temporary(longitude), $
                scan : temporary(scan), $
                sequence : temporary(sequence), $
                CloudFrequency : temporary(CloudFrequency),$
                EmissivityWavenumber : temporary(EmissivityWavenumber), $
                pressuregrid : temporary(pressuregrid), $
                Scan_Resolution : temporary(Scan_Resolution), $
                VerticalCoordinate : temporary(VerticalCoordinate) $
                }
    h5g_close,specid
endif



;=====================================================================
swath = 'CO2NadirSwath'
;=====================================================================
swpos = where(rswaths eq swath)
if swpos ne -1 then begin
; data fields
    specid = h5g_open(swid,swath)
    fieldid = h5g_open(specid,'Data Fields')
;
; actual data, alphabetically except for the gas
;
; get name from swath name
    lpos = strpos(swath,'Limb')
    npos = strpos(swath,'Nadir')
    if lpos ne -1 then gas = strmid(swath,0,lpos)
    if npos ne -1 then gas = strmid(swath,0,npos)
    
    dataid = h5d_open(fieldid,gas)
    species = h5d_read(dataid)
    ;get_data_attributes,dataid,size(species,/type),attrib0
    h5d_close,dataid


    dataid = h5d_open(fieldid,'Pressure')
    Pressure = h5d_read(dataid)
    get_data_attributes,dataid,size(Pressure,/type),attrib26
    h5d_close,dataid

    h5g_close,fieldid
    geoid = h5g_open(specid,'Geolocation Fields')

    dataid = h5d_open(geoid,'Latitude')
    Latitude = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Latitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'Longitude')
    Longitude = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Longitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'Scan')
    Scan = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Scan,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'Sequence')
    Sequence = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Sequence,/type),attrib7
    h5d_close,dataid

    h5g_close,geoid
;
; get swath level attributes
;
    id = h5a_open_name(specid,'Pressure')
    pressuregrid = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'VerticalCoordinate')
    VerticalCoordinate = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'CloudFrequency')
    CloudFrequency = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'EmissivityWavenumber')
    EmissivityWavenumber = h5a_read(id)
    h5a_close,id


    id = h5a_open_name(specid,'Scan_Resolution')
    Scan_Resolution = h5a_read(id)
    h5a_close,id

    co2nadir = { $
                species :  temporary(species), $
                Pressure : temporary(Pressure), $
                latitude : temporary(latitude), $
                longitude : temporary(longitude), $
                scan : temporary(scan), $
                sequence : temporary(sequence), $
                CloudFrequency : temporary(CloudFrequency),$
                EmissivityWavenumber : temporary(EmissivityWavenumber), $
                pressuregrid : temporary(pressuregrid), $
                Scan_Resolution : temporary(Scan_Resolution), $
                VerticalCoordinate : temporary(VerticalCoordinate) $
                }
    h5g_close,specid
endif



;=====================================================================
swath = 'N2OLimbSwath'
;=====================================================================
swpos = where(rswaths eq swath)
if swpos ne -1 then begin
; data fields
    specid = h5g_open(swid,swath)
    fieldid = h5g_open(specid,'Data Fields')
;
; actual data, alphabetically except for the gas
;
; get name from swath name
    lpos = strpos(swath,'Limb')
    npos = strpos(swath,'Nadir')
    if lpos ne -1 then gas = strmid(swath,0,lpos)
    if npos ne -1 then gas = strmid(swath,0,npos)
    
    dataid = h5d_open(fieldid,gas)
    species = h5d_read(dataid)
    ;get_data_attributes,dataid,size(species,/type),attrib0
    h5d_close,dataid


    dataid = h5d_open(fieldid,'Pressure')
    Pressure = h5d_read(dataid)
    get_data_attributes,dataid,size(Pressure,/type),attrib26
    h5d_close,dataid

    h5g_close,fieldid
    geoid = h5g_open(specid,'Geolocation Fields')

    dataid = h5d_open(geoid,'Latitude')
    Latitude = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Latitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'Longitude')
    Longitude = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Longitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'Scan')
    Scan = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Scan,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'Sequence')
    Sequence = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Sequence,/type),attrib7
    h5d_close,dataid

    h5g_close,geoid
;
; get swath level attributes
;
    id = h5a_open_name(specid,'Pressure')
    pressuregrid = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'VerticalCoordinate')
    VerticalCoordinate = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'CloudFrequency')
    CloudFrequency = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'EmissivityWavenumber')
    EmissivityWavenumber = h5a_read(id)
    h5a_close,id


    id = h5a_open_name(specid,'Scan_Resolution')
    Scan_Resolution = h5a_read(id)
    h5a_close,id

    n2olimb = { $
                species :  temporary(species), $
                Pressure : temporary(Pressure), $
                latitude : temporary(latitude), $
                longitude : temporary(longitude), $
                scan : temporary(scan), $
                sequence : temporary(sequence), $
                CloudFrequency : temporary(CloudFrequency),$
                EmissivityWavenumber : temporary(EmissivityWavenumber), $
                pressuregrid : temporary(pressuregrid), $
                Scan_Resolution : temporary(Scan_Resolution), $
                VerticalCoordinate : temporary(VerticalCoordinate) $
                }
    h5g_close,specid
endif



;=====================================================================
swath = 'N2ONadirSwath'
;=====================================================================
swpos = where(rswaths eq swath)
if swpos ne -1 then begin
; data fields
    specid = h5g_open(swid,swath)
    fieldid = h5g_open(specid,'Data Fields')
;
; actual data, alphabetically except for the gas
;
; get name from swath name
    lpos = strpos(swath,'Limb')
    npos = strpos(swath,'Nadir')
    if lpos ne -1 then gas = strmid(swath,0,lpos)
    if npos ne -1 then gas = strmid(swath,0,npos)
    
    dataid = h5d_open(fieldid,gas)
    species = h5d_read(dataid)
    ;get_data_attributes,dataid,size(species,/type),attrib0
    h5d_close,dataid


    dataid = h5d_open(fieldid,'Pressure')
    Pressure = h5d_read(dataid)
    get_data_attributes,dataid,size(Pressure,/type),attrib26
    h5d_close,dataid

    h5g_close,fieldid
    geoid = h5g_open(specid,'Geolocation Fields')

    dataid = h5d_open(geoid,'Latitude')
    Latitude = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Latitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'Longitude')
    Longitude = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Longitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'Scan')
    Scan = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Scan,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'Sequence')
    Sequence = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Sequence,/type),attrib7
    h5d_close,dataid

    h5g_close,geoid
;
; get swath level attributes
;
    id = h5a_open_name(specid,'Pressure')
    pressuregrid = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'VerticalCoordinate')
    VerticalCoordinate = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'CloudFrequency')
    CloudFrequency = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'EmissivityWavenumber')
    EmissivityWavenumber = h5a_read(id)
    h5a_close,id


    id = h5a_open_name(specid,'Scan_Resolution')
    Scan_Resolution = h5a_read(id)
    h5a_close,id

    n2onadir = { $
                species :  temporary(species), $
                Pressure : temporary(Pressure), $
                latitude : temporary(latitude), $
                longitude : temporary(longitude), $
                scan : temporary(scan), $
                sequence : temporary(sequence), $
                CloudFrequency : temporary(CloudFrequency),$
                EmissivityWavenumber : temporary(EmissivityWavenumber), $
                pressuregrid : temporary(pressuregrid), $
                Scan_Resolution : temporary(Scan_Resolution), $
                VerticalCoordinate : temporary(VerticalCoordinate) $
                }
    h5g_close,specid
endif



;=====================================================================
swath = 'O3LimbSwath'
;=====================================================================
swpos = where(rswaths eq swath)
if swpos ne -1 then begin
; data fields
    specid = h5g_open(swid,swath)
    fieldid = h5g_open(specid,'Data Fields')
;
; actual data, alphabetically except for the gas
;
; get name from swath name
    lpos = strpos(swath,'Limb')
    npos = strpos(swath,'Nadir')
    if lpos ne -1 then gas = strmid(swath,0,lpos)
    if npos ne -1 then gas = strmid(swath,0,npos)
    
    dataid = h5d_open(fieldid,gas)
    species = h5d_read(dataid)
    ;get_data_attributes,dataid,size(species,/type),attrib0
    h5d_close,dataid


    dataid = h5d_open(fieldid,'Pressure')
    Pressure = h5d_read(dataid)
    get_data_attributes,dataid,size(Pressure,/type),attrib26
    h5d_close,dataid

    h5g_close,fieldid
    geoid = h5g_open(specid,'Geolocation Fields')

    dataid = h5d_open(geoid,'Latitude')
    Latitude = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Latitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'Longitude')
    Longitude = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Longitude,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'Scan')
    Scan = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Scan,/type),attrib7
    h5d_close,dataid

    dataid = h5d_open(geoid,'Sequence')
    Sequence = h5d_read(dataid)
    ;get_data_attributes,dataid,size(Sequence,/type),attrib7
    h5d_close,dataid

    h5g_close,geoid
;
; get swath level attributes
;
    id = h5a_open_name(specid,'Pressure')
    pressuregrid = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'VerticalCoordinate')
    VerticalCoordinate = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'CloudFrequency')
    CloudFrequency = h5a_read(id)
    h5a_close,id

    id = h5a_open_name(specid,'EmissivityWavenumber')
    EmissivityWavenumber = h5a_read(id)
    h5a_close,id


    id = h5a_open_name(specid,'Scan_Resolution')
    Scan_Resolution = h5a_read(id)
    h5a_close,id

    o3limb = { $
             species :  temporary(species), $
             Pressure : temporary(Pressure), $
             latitude : temporary(latitude), $
             longitude : temporary(longitude), $
             scan : temporary(scan), $
             sequence : temporary(sequence), $
             CloudFrequency : temporary(CloudFrequency),$
             EmissivityWavenumber : temporary(EmissivityWavenumber), $
             pressuregrid : temporary(pressuregrid), $
             Scan_Resolution : temporary(Scan_Resolution), $
             VerticalCoordinate : temporary(VerticalCoordinate) $
             }
    h5g_close,specid
endif


h5g_close,swid




;
; get 'Additional' information
;
aid = h5g_open(hdfid,'ADDITIONAL')
faaid = h5g_open(aid,'FILE_ATTRIBUTES')

if read_ph ne 0 then begin
    dataid = h5d_open(faaid,'ProductionHistory')
    ProductionHistory = h5d_read(dataid)
    h5d_close,dataid
endif else begin
    ProductionHistory = ''
endelse

id = h5a_open_name(faaid,'InstrumentName')
InstrumentName = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'ProcessLevel')
ProcessLevel = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'TAI93At0zOfGranule')
TAI93At0zOfGranule = h5a_read(id)
    h5a_close,id
 
id = h5a_open_name(faaid,'GlobalSurveyNumber')
GlobalSurveyNumber = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'GranuleMonth')
GranuleMonth = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'GranuleDay')
GranuleDay = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'GranuleYear')
GranuleYear = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'SurveyMode')
SurveyMode = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'PGEVersion')
PGEVersion = h5a_read(id)
h5a_close,id

id = h5a_open_name(faaid,'Command_Seq_ID')
Command_Seq_ID = h5a_read(id)
h5a_close,id
  


h5g_close,faaid
h5g_close,aid
h5g_close,hdfid

    file_attributes = { $
                  ProductionHistory : temporary(ProductionHistory), $
                  InstrumentName : temporary(InstrumentName), $
                  ProcessLevel : temporary(ProcessLevel), $
                  TAI93At0zOfGranule : temporary(TAI93At0zOfGranule), $
                  GlobalSurveyNumber : temporary(GlobalSurveyNumber), $
                  GranuleMonth : temporary(GranuleMonth), $
                  GranuleDay : temporary(GranuleDay), $
                  GranuleYear : temporary(GranuleYear), $
                  SurveyMode : temporary(SurveyMode), $
                  PGEVersion : temporary(PGEVersion), $
                  Command_Seq_ID : temporary(Command_Seq_ID)    }




hdfinfid = h5g_open(fid,'HDFEOS INFORMATION')

dataid = h5d_open(hdfinfid,'StructMetadata.0')
StructMetadata = h5d_read(dataid)
h5d_close,dataid

dataid = h5d_open(hdfinfid,'coremetadata')
coremetadata = h5d_read(dataid)
h5d_close,dataid

id = h5a_open_name(hdfinfid,'HDFEOSVersion')
HDFEOSVersion = h5a_read(id)
h5a_close,id

h5g_close,hdfinfid
h5f_close,fid

hdfeos_data = { $
              StructMetadata : temporary(StructMetadata), $
              coremetadata : temporary(coremetadata), $
              HDFEOSVersion : temporary(HDFEOSVersion) }


return

end
